/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/adc_stm32.h"           */
/*  Declaration of the procedures of the DACs'.                              */
/*                                                                           */
/*  Author : JYA                        Last modification : 10/07/2020       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef ADC_STM32_H                    /* If this file is not already        */
#define ADC_STM32_H                    /* included, include it.              */


/*
    1    Defines
    ____________
*/


/*
    2    Structures and typedefs
    ____________________________
*/

typedef struct                         /* Parametres du device ADC           */
  BEGIN                                /* ---------------------------------- */
    UCHAR         adc_num            ; /* ADC number                         */
    CHAR         *tim_name           ; /* Timer name                         */
    UCHAR         dma_num            ; /* DMA number                         */
    UCHAR         trig               ; /* External trigger index             */
    UCHAR         dma_sdn            ; /* DMA sub-device number              */
  END             Adc_stm32_def      ; /* ---------------------------------- */

typedef struct                         /* Low level Context                  */
  BEGIN                                /* Constant values ------------------ */
    UCHAR            dev             ; /* Device num. for the generic ADC drv*/
    UCHAR            adc_num         ; /* ADC number                         */
    UCHAR            tim_num         ; /* Timer number                       */
    UCHAR            dma_num         ; /* DMA number                         */
    UCHAR            dma_sdn         ; /* DMA sub-device number              */
    UCHAR            trig            ; /* External trigger index             */
    volatile Adc    *creg            ; /* ADC common registers base address  */
    volatile AdcNum *areg            ; /* ADC dedicated registers base addr  */
                                       /*                                    */
                                       /* Almost constant values ----------- */
    UINT             smp_prd         ; /* Sampling period in number of       */
                                       /* machine cycles                     */
    UCHAR            dep             ; /* Sample depth                       */
    UCHAR            dma             ; /* 0: FIFO, 1: DMA                    */
                                       /*                                    */
                                       /* For DMA transfers ---------------- */
    UCHAR            hit             ; /* 0: transfer complete interrupt     */
                                       /* 1: half transfer interrupt         */
    UCHAR           *plb             ; /* Pointer in local buffer used by DMA*/
    UCHAR           *pab             ; /* Address of the buffer sent by appli*/
    INT              sz              ; /* Size of the buffer sent by appli   */
    INT              ind             ; /* Index in the buffer sent by appli  */
                                       /*                                    */
                                       /* For the timer -------------------- */
//    UCHAR         jext_trigg         ; /* external trig for injected channel */
//    UCHAR         accu_ch            ; /* accurate channel number            */
//    UINT          width              ; /* Width                              */
//    UINT          wmask              ; /* mask associated to the width       */
//    volatile Tim *timaddr            ; /* Timer address                      */
//    UINT          freq               ; /* Timer frequency                    */
                                       /*                                    */
                                       /* Error counter -------------------- */
    UINT             cnr_ovr         ; /* Overrun error counter              */
  END                Adc_stm32       ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


extern const Adc_desc adc_desc_stm32 ; /* ADC low level interface            */
extern       Adc_stm32 adc_stm32[]   ; /* Context table of tim_stm32.c       */


/*
    4    Prototypes of public procedures
    ____________________________________
*/


/* Declaration of the procedures ........................................... */

IWORD handlerADC_stm32                 (IWORD,IWORD                          );



#endif                                 /* End of ADC_STM32_H                 */
