/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/asy_stm32.h"           */
/*  Declaration of the procedures of the UARTs'.                             */
/*                                                                           */
/*  Author : JYA                        Last modification : 11/23/2018       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef ASY_STM32_H                    /* If this file is not already        */
#define ASY_STM32_H                    /* included, include it.              */




/*
    1    Defines
    ____________
*/



/*
    2    Structures and typedefs
    ____________________________
*/

typedef struct                         /* Control signals GPIO's             */
  BEGIN                                /* ---------------------------------- */
    CHAR               *uname        ; /* GPIO user name                     */
    UCHAR              inv           ; /* 0:Normal 1:Inverted                */
  END                  Asy_stm32_sig ; /* ---------------------------------- */

typedef struct                         /* Parametres du driver ASY           */
  BEGIN                                /* ---------------------------------- */
    UCHAR              asc_num       ; /* Numero du controleur ASC.          */
    CHAR              *tim_name      ; /* Timer name                         */
    UCHAR              dma_num       ; /* Numero du DMA                      */
    UCHAR              tx_sdev       ; /* DMA sub-device number for TX       */
    UCHAR              rx_sdev       ; /* DMA sub-device number for RX       */
    Asy_stm32_sig      sig[8]        ; /* 3 bytes :                          */
  END                  Asy_stm32_def ; /* ---------------------------------- */

typedef struct                         /* Low-level context                  */
  BEGIN                                /* Constant values ------------------ */
    IWORD              dev           ; /* Device number for the generic drv  */
    UCHAR              uart_num      ; /* UART  number                       */
    UCHAR              tim_num       ; /* Timer number                       */
    UCHAR              dma_num       ; /* DMA   number                       */
    UCHAR              tx_sdev       ; /* DMA sub-device number for TX       */
    UCHAR              rx_sdev       ; /* DMA sub-device number for RX       */
    UCHAR              pio[8*3]      ; /* Signal description table           */
    volatile Asc       *uart_reg     ; /* UART registers base address        */
                                       /*                                    */
                                       /* Almost constant values ----------- */
    UCHAR              dma           ; /* 0: FIFO, 1: DMA                    */
    UCHAR              tim_mode      ; /* Timer 0:NONE 1:CHARACTER 2:FRAME   */
    UCHAR              ubits         ; /* Count of data bits per character   */
    UCHAR              rcvdma        ; /* 0:IDLE 1:Circ receive DMA started  */
                                       /*                                    */
                                       /* For DMA transfers ---------------- */
    UCHAR              *circbuf      ; /* Receive circular buf address       */
    USHORT             circsz        ; /* Receive circular buf size in bytes */
    UCHAR              *userbuf      ; /* User buffer address                */
    USHORT             usersz        ; /* User buffer size in bytes          */
    USHORT             userlen       ; /* Already written bytes in userbuf   */
                                       /*                                    */
                                       /* For the timer -------------------- */
    UINT               baud_rate     ; /* Baud rate                          */
    UINT               to_clk        ; /* Timeout value in milliseconds      */
                                       /*                                    */
    UINT               extra_tx      ; /* Backup of the XON/XOFF             */
                                       /* Error counters ------------------- */
    USHORT             cnr_pe        ; /* Counter "parity error"             */
    USHORT             cnr_fe        ; /* Counter "frame error"              */
    USHORT             cnr_nf        ; /* Counter "noise detected flag"      */
    USHORT             cnr_ore       ; /* Counter "overrun error"            */
  END                  Asy_stm32     ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/

extern const Asy_desc asy_desc_stm32 ; /* ASY  low level interface           */
extern       Asy_stm32 asy_stm32[]   ; /* Context table of tim_stm32.c       */

/*
    4    Prototypes of public procedures
    ____________________________________
*/

/* Declaration of the procedures ........................................... */

IWORD handlerTOASY_stm32               (IWORD,IWORD                          );
IWORD handlerASY_stm32                 (IWORD,IWORD                          );
IWORD handlerdmaASY_stm32              (IWORD,IWORD                          );


#endif                                 /* End of ASY_STM32_H                 */
