/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/Dma_stm32.h"           */
/*  Declaration of the procedures of the STM32 DMA.                          */
/*                                                                           */
/*  Author : JYA                        Last modification : 08/18/2020       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef DMA_STM32_H                    /* If this file is not already        */
#define DMA_STM32_H                    /* included, include it.              */




/*
    1    Defines
    ____________
*/

/* DMA channel x configuration register .....................................*/

#define CCR_EN          0x0001         /* Channel enable                     */
#ifdef VM_STM32M3
  #define CCR_TCIE      0x0002         /* Transfer complete interrupt enable */
  #define CCR_HTIE      0x0004         /* Half transfer interrupt enable     */
  #define CCR_TEIE      0x0008         /* Transfer error interrupt enable    */
  #define CCR_DIR       0x0010         /* Data transfer direction            */
  #define CCR_CIRC      0x0020         /*             */
  #define CCR_PINC      0x0040         /*             */
  #define CCR_MINC      0x0080         /*             */
  #define CCR_PSIZE     0x0300         /*             */

  #define CCR_PSIZE_8   0x0000         /* Peripheral size =  8-bits          */
  #define CCR_PSIZE_16  0x0100         /* Peripheral size = 16-bits          */
  #define CCR_PSIZE_32  0x0200         /* Peripheral size = 32-bits          */

  #define CCR_MSIZE     0x0C00         /* Memory size                        */
  #define CCR_MSIZE_8   0x0000         /* Memory size =  8-bits              */
  #define CCR_MSIZE_16  0x0400         /* Memory size = 16-bits              */
  #define CCR_MSIZE_32  0x0800         /* Memory size = 32-bits              */

  #define CCR_PL        0x3000         /*             */
  #define CCR_PL_LOW    0x0000         /*             */
  #define CCR_PL_MEDIUM 0x1000         /*             */
  #define CCR_PL_HIGH   0x2000         /*             */
  #define CCR_PL_VHIGHT 0X3000         /*             */

  #define CCR_MEM2MEM   0x4000         /*             */
#endif

#ifdef VM_STM32M4
  #define CCR_PSIZE_8   0x0000         /* Peripheral size =  8-bits          */
  #define CCR_PSIZE_16  0x0800         /* Peripheral size = 16-bits          */
  #define CCR_PSIZE_32  0x1000         /* Peripheral size = 32-bits          */

  #define CCR_MSIZE     0x6000         /* Memory size                        */
  #define CCR_MSIZE_8   0x0000         /* Memory size =  8-bits              */
  #define CCR_MSIZE_16  0x2000         /* Memory size = 16-bits              */
  #define CCR_MSIZE_32  0x4000         /* Memory size = 32-bits              */
#endif


/* DMA interrupt status register ............................................*/

#ifdef VM_STM32M3
  #define ISR_GIF       0x0001         /* Channel x global interrupt flag    */
  #define ISR_TCIF      0x0002         /* Channel x transfer complete flag   */
  #define ISR_HTIF      0x0004         /* Channel x half transfer flag       */
  #define ISR_TEIF      0x0008         /* Channel x transfer error flag      */
  #define ISR_HTIF_TCIF 0x0006         /* Channel x half transfer and        */
                                       /* transfer complete flags            */
  #define ISR_CHX       0x000F         /* Channel x flags                    */
#endif
#ifdef VM_STM32M4
  #define ISR_FEIF      0x0001         /* Stream x FIFO error interrupt flag */
  #define ISR_DMEIF     0x0004         /* Stream x direct mode error IT flag */
  #define ISR_TEIF      0x0008         /* Stream x transfer error IT flag    */
  #define ISR_HTIF      0x0010         /* Stream x half transfer IT flag     */
  #define ISR_TCIF      0x0020         /* Stream x transfer complete IT flag */
  #define ISR_HTIF_TCIF 0x0030         /* Stream x half transfer and transfer*/
                                       /* complete flags                     */
  #define ISR_STX       0x003D         /* Stream x flags                     */
#endif


/* DMA interrupt flag clear register ........................................*/

#ifdef VM_STM32M3
  #define IFCR_CGIF     0x0001         /* Channel x global interrupt clear   */
  #define IFCR_CTCIF    0x0002         /* Channel x transfer complete clear  */
  #define IFCR_CHTIF    0x0004         /* Channel x half transfer flag       */
  #define IFCR_CTEIF    0x0008         /* Channel x transfer error clear     */
  #define IFCR_CHX      0x000F         /* Channel x clears                   */
#endif

#ifdef VM_STM32M4
  #define IFCR_CFEIF    0x0001         /* Stream x clear FIFO error interrupt*/
  #define IFCR_CDMEIF   0x0004         /* Stream x clear direct mode error IT*/
  #define IFCR_CTEIF    0x0008         /* Stream x clear transfer error IT   */
  #define IFCR_CHTIF    0x0010         /* Stream x clear half transfer IT    */
  #define IFCR_CTCIF    0x0020         /* Stream x clear transfer complete IT*/
  #define IFCR_STX      0x003D         /* Stream x clears                    */
#endif


/*
    2    Structures and typedefs
    ____________________________
*/

typedef struct                         /* Parameters of the device DMA       */
  BEGIN                                /* ---------------------------------- */
    UCHAR            strnum[8]       ; /* Stream number                      */
  END                Dma_stm32_def   ; /* ---------------------------------- */

typedef struct                         /* Low-level DMA channel context      */
  BEGIN                                /* ---------------------------------- */
    USHORT           chanidx         ; /* High level context index           */
    UCHAR            chsta           ; /* Channel state variable             */
    UCHAR            strnum          ; /* Stream number                      */
    Dma_itproc       *itcb           ; /* Call-back procedure struct adr     */
    volatile DmaSdev *chreg          ; /* Dedicated channel registers addr   */
    UCHAR            *ciradr         ; /* Circular buffer start address      */
    USHORT           cirlen          ; /* Circular buffer length in size     */
    USHORT           cirwr           ; /* Next byte to be written offset     */
    USHORT           cirrd           ; /* Next byte to be read offset        */
    USHORT           cirnb           ; /* Cout of available bytes            */
    UCHAR            cirdsz          ; /* Elementary transfer size (1/2/4)   */
  END                Dma_stm32_ch    ; /* ---------------------------------- */

typedef struct                         /* Low-level DMA context structure    */
  BEGIN                                /* ---------------------------------- */
    UCHAR            state           ; /* 0:Not init      1:Initialized      */
    Dma_stm32_ch     hch32[8]        ; /* 8 hardware channel context         */
  END                Dma_stm32       ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/

extern const Dma_desc dma_desc_stm32 ; /* DMA low level interface            */


/*
    4    Prototypes of public procedures
    ____________________________________
*/

/* Declaration of the procedures ........................................... */

IWORD  infoDMA_stm32          (INT,CHAR**,INT*                               );
IWORD  initDMA_stm32          (INT                                           );
IWORD  endDMA_stm32           (INT                                           );
IWORD  init_hchanDMA_stm32    (INT,INT,INT,INT                               );
IWORD  end_hchanDMA_stm32     (INT,INT                                       );
IWORD  abortDMA_stm32         (INT,INT                                       );
IWORD  irqDMA_stm32           (INT,INT,INT                                   );
IWORD  setitcbDMA_stm32       (INT,INT,Dma_itcb*                             );

IWORD  sndDMA_stm32           (INT,INT,UCHAR*,INT,UCHAR*,INT,INT,INT         );
IWORD  rcvDMA_stm32           (INT,INT,UCHAR*,INT,UCHAR*,INT,INT,INT         );
IWORD  cpyDMA_stm32           (INT,INT,UCHAR*,UCHAR*,INT                     );
IWORD  begaccDMA_stm32        (INT,INT,INT,UCHAR**,INT*,INT*                 );
IWORD  endaccDMA_stm32        (INT,INT,INT,INT                               );

IWORD  sndSLOT_stm32          (INT,INT,UCHAR*,INT,INT,INT                    );
IWORD  rcvSLOT_stm32          (INT,INT,UCHAR*,INT,INT,INT                    );
IWORD  setSLOT_stm32          (INT,INT,INT,INT,UCHAR*,INT                    );
IWORD  getSLOT_stm32          (INT,INT,INT,INT*,UCHAR**,INT*                 );

IWORD  handlerDMA_stm32       (INT,INT                                       );


#endif                                 /* End of DMA_STM32_H                 */
