/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/genio.h"               */
/*  Constants and definitions for drivers and devices API.                   */
/*                                                                           */
/*  Author : JYA                        Last modification : 13/04/2017       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
            2.1  Contexte et description de POWER
            2.2  Contexte et description de KEYBOARD
            2.3  Contexte et description de bus SPI
            2.4  Contexte et description de CONSOLE
            2.5  Contexte et description de ATA
            2.6  Contexte et description de FAN
            2.7  Contexte et description de NAND
            2.8  Contexte et description de ETHERNET/ARP/FAKE
            2.9  Contexte et description de PLAY
            2.10 Contexte et description de GLUE
            2.11 Contexte et description de HTTP
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef GENIO_H                        /* If this file is not already        */
#define GENIO_H                        /* included, include it.              */


/*
    1    Defines
    ____________
*/


/* Types de flash definis par le driver dsk_nand.c ..........................*/

#define NAND_STA              0        /* ST type A like                     */
#define NAND_STB              1        /* ST type B like                     */
#define NAND_SAMSUNG          2        /* Samsung like                       */


/* Description de ETHERNET/ARP/FAKE .........................................*/

#define ERR_NOLINK      (INT)-1        /* Le cable Ethernet est debranche.   */
#define COLLISION       (INT)-2        /* Trop de collisions a l'emission.   */
#define ERR_FIFO_FULL   (INT)-3        /* File pleine.                       */
#define RESOLVE_ERROR   (INT)-4        /* Echec de la resolution ARP.        */

#define SZ_NET_HDR           20        /* Taille min a laisser pour header   */
#define SZ_PRO_ADR           16        /* Taille max de adresse protocolaire */

#ifndef ETH_NBRX                       /*                                    */
#define ETH_NBRX             64        /* Nb max de IT_RX d'affilee.         */
#endif                                 /*                                    */

#define ETH_NBTX             64        /* Nb max de buffer chaines en TX     */
#define ETH_NBUNODE           8        /* Nb max de nodes utilisateurs.      */
#define ETH_NBSNODE           8        /* Nb max de nodes systeme.           */

#define ETH_NBNODE    (ETH_NBUNODE+ETH_NBSNODE) /* Nb max de nodes           */

#define ETH_DELAYQ            4        /* 4 slots d'attente                  */
#define ETH_BWLIM            10        /* Mbps                               */

#define ETH_NOLINK            0        /* Pas de branchement Eth             */
#define ETH_10            (1<<0)       /* 10 Base                            */
#define ETH_100           (2<<0)       /* 100 Base                           */
#define ETH_1G            (3<<0)       /* Gigabit Ethernet                   */
#define ETH_FULLDUPLEX    (1<<2)       /* Mode Full duplex                   */
#define ETH_FLOWCTL       (1<<3)       /* Controle de flux                   */
#define ETH_CONFLICT      (1<<4)       /* Gestion de conflit ARP             */

#define ARP_SZCACHE          16        /*                                    */
#define ARP_NBJOB             8        /*                                    */

#define M_READ                1        /* Si on recoit des REQ_READ          */
#define M_IFBIND              2        /* Si on recoit REQ_IFSTART/UNBIND    */
#define M_IFSTART             4        /* Si on recoit REQ_IFSTART/IFSTOP    */
#define SZ_PRO_ADR           16        /* Taille max de adresse protocolaire */

#define IT_CON1               3        /*                                    */
#define IT_CON2               4        /*                                    */
#define IT_DIS0               5        /*                                    */
#define IT_DIS1               6        /*                                    */
#define IT_DIS2               7        /*                                    */


/* GLUE - Tailles des registres de type Chaine de caracteres ................*/

#define NBRGLUETYPE           2        /* Nb UINTs registres NRTYPE.         */
#define NBRGLUENAME           2        /* Nb UINTs registres NRNAME.         */
#define NBRGLUEFILEVOL  (256/ZI)       /* Nb UINTs registres NRFILEVOL.      */
#define NBRGLUEFILEPATH (256/ZI)       /* Nb UINTs registres NRFILEPATH.     */


/* GLUE - Numeros des registres .............................................*/

#define GLUERTYPE         0                                   /*           0 */
#define GLUERNAME         (GLUERTYPE        +NBRGLUETYPE    ) /*   0+ 2 =  2 */
#define GLUERIODPLAY      (GLUERNAME        +NBRGLUENAME    ) /*   2+ 2 =  4 */
#define GLUERFILEVOL      (GLUERIODPLAY     +1              ) /*   4+ 1 =  5 */
#define GLUERFILEPATH     (GLUERFILEVOL     +NBRGLUEFILEVOL ) /*   5+64 = 69 */
#define GLUERNBYTES       (GLUERFILEPATH    +NBRGLUEFILEPATH) /*  69+64 =133 */
#define GLUERSTATUS       (GLUERNBYTES      +1              ) /* 133+ 1 =134 */
#define GLUE_NBREG        (GLUERSTATUS      +1              ) /* 134+ 1 =135 */


/* HTTP - Tailles des registres de type Chaine de caracteres ................*/

#define NBRHTTPTYPE           2        /* Nb UINTs registres NRTYPE.         */
#define NBRHTTPNAME           2        /* Nb UINTs registres NRNAME.         */
#define NBRHTTPUSER      (80/ZI)       /* Nb UINTs registres NRUSER.         */
#define NBRHTTPMIME      (32/ZI)       /* Nb UINTs registres NRMIME.         */
#define NBRHTTPACCEPT    (80/ZI)       /* Nb UINTs registres NRACCEPT.       */
#define NBRHTTPPHRASE    (32/ZI)       /* Nb UINTs registres NRPHRASE.       */


/* HTTP - Numeros des registres .............................................*/

#define HTTPRTYPE             0
#define HTTPRNAME        (HTTPRTYPE   + NBRHTTPTYPE  )
#define HTTPRUSER        (HTTPRNAME   + NBRHTTPNAME  )
#define HTTPRACCEPT      (HTTPRUSER   + NBRHTTPUSER  )
#define HTTPRMIME        (HTTPRACCEPT + NBRHTTPACCEPT)
#define HTTPRLENGTH      (HTTPRMIME   + NBRHTTPMIME  )
#define HTTPRLEVEL       (HTTPRLENGTH + 1            )
#define HTTPRIPSRV       (HTTPRLEVEL  + 1            )
#define HTTPRMETHOD      (HTTPRIPSRV  + 1            )
#define HTTPRCODE        (HTTPRMETHOD + 1            )
#define HTTPRPHRASE      (HTTPRCODE   + 1            )
#define HTTPRCMD         (HTTPRPHRASE + NBRHTTPPHRASE)
#define NBREG_HTTP       (HTTPRCMD    + 1            )


/* HTTP - Valeurs des tags enumeres .........................................*/

#define MGET                  0        /* METHOD=GET                         */
#define MHEAD                 1        /* METHOD=HEAD                        */
#define MPOST                 2        /* METHOD=POST                        */

#define LEVEL_FULL            0        /* LEVEL=FULLAUTO                     */
#define LEVEL_DNS             1        /* LEVEL=DNSAUTO                      */
#define LEVEL_MANU            2        /* LEVEL=MANUAL                       */


/* Mnemoniques de DSK .......................................................*/

#define MUID0              R123        /* "vmos_memuid"                      */
#define MUID1              R124        /* "dsk_memuid"                       */


/*
    2    Structures and typedefs
    ____________________________
*/


/*
    2.1 Contexte et description de POWER
    ------------------------------------
*/
 

typedef struct                         /* Device POWER                       */
  BEGIN                                /* ---------------------------------- */
    void            (*info)()        ; /* Valeurs des registres.             */
    IWORD           (*set )()        ; /* Iod du contexte.                   */
  END               Pow_desc         ; /* ---------------------------------- */
                                                                             
typedef struct                         /* Contexte d'un device POWER         */
  BEGIN                                /* ---------------------------------- */
    UINT            reg[8]           ; /* Valeurs des registres.             */
    UINT            iod              ; /* Iod du contexte.                   */
    UINT            to               ; /* Temporisation rappel bas niveau    */
    USHORT          openemet         ; /* Num automate emetteur REQ_OPEN     */
    USHORT          openevl          ; /* VL emettrice REQ_OPEN              */
    USHORT          openid           ; /* Id utilisateur requete REQ_OPEN    */
    USHORT          reqemet          ; /* Num automate emetteur REQ_SETVAL   */
    USHORT          reqevl           ; /* VL emettrice REQ_SETVAL            */
    USHORT          reqid            ; /* Id. utilisateur REQ_SETVAL         */
    USHORT          sessid           ; /* Numero de session                  */
  END               Pow_ctx          ; /* ---------------------------------- */


/*
    2.2 Contexte et description de KEYBOARD
    ---------------------------------------
*/


typedef struct                         /* Device KBD                         */
  BEGIN                                /* ---------------------------------- */
    IWORD          (*init_keyboard)(); /* Keyboard initialization function   */
    void           (*end_keyboard)() ; /* Keyboard termination function      */
  END               Kbd_desc         ; /* ---------------------------------- */


/*
    2.3 Contexte et description de bus SPI
    --------------------------------------
*/


typedef struct                         /* Descr. driver bas niveau SPI       */
  BEGIN                                /* ---------------------------------- */
    IWORD          (*start)   ()     ; /* Demarrer transaction               */
    IWORD          (*stop )   ()     ; /* Arreter  transaction               */
    IWORD          (*io   )   ()     ; /* Effectuer cycle(s)                 */
  END               Spi_desc         ; /* ---------------------------------- */


/*
    2.4 Contexte et description de CONSOLE
    --------------------------------------
*/


typedef struct                         /* Device CONSOLE                     */
  BEGIN                                /* ---------------------------------- */
    void           (*open)    ()     ; /* Initialision                       */
    void           (*close)   ()     ; /* Fermeture                          */
    void           (*put)     ()     ; /* Ecriture sur la console            */
    void           (*get)     ()     ; /* Lecture  sur le console            */
    CHAR           *name             ; /* Nom du device                      */
  END               Cons_desc        ; /* ---------------------------------- */

typedef struct                         /* Contexte I2C                       */
  BEGIN                                /* ---------------------------------- */
    void           (*open)    ()     ; /* Initialision                       */
    void           (*close)   ()     ; /* Fermeture                          */
    void           (*put)     ()     ; /* Ecriture sur la console            */
    void           (*get)     ()     ; /* Lecture  sur le console            */
    INT             ldev             ; /* Numero device driver bas niveau    */
    INT             bauds            ; /* Vitesse de ligne en bauds          */
    INT             state            ; /* 0:Ferme 1:ouvert                   */
  END               Cons_ctx         ; /* ---------------------------------- */


/*
    2.5 Contexte et description de ATA
    ----------------------------------
*/


typedef struct                         /* Descr. driver bas niveau ATA       */
  BEGIN                                /* ---------------------------------- */
    IWORD          (*info        ) (); /* Obtention vitesse max autorisee    */
    IWORD          (*reset_ctr   ) (); /* Reset hardware d'un controleur     */
    IWORD          (*detect_dev  ) (); /* Detection des devices presents     */
    IWORD          (*enable_int  ) (); /* Autoriser les ITs du controleur    */
    IWORD          (*set_mode    ) (); /* Regler vitesse commmunication      */
    IWORD          (*reset_dev   ) (); /* Reset hardware des devices         */
    IWORD          (*poll        ) (); /* Polling du registre Status         */
    IWORD          (*set_cmd_reg ) (); /* Ecrire registres du bloc COMMAND   */
    IWORD          (*get_cmd_reg ) (); /* Lire   registres du bloc COMMAND   */
    IWORD          (*set_ctr_reg ) (); /* Ecrire registre  du bloc CONTROL   */
    IWORD          (*get_ctr_reg ) (); /* Lire   registre  du bloc CONTROL   */
    IWORD          (*set_data_reg) (); /* Ecrire sect. dans registre DATA    */
    IWORD          (*get_data_reg) (); /* Lire   sect. dans registre DATA    */
    IWORD          (*start_wdma  ) (); /* Init. transfert DMA vers disque    */
    IWORD          (*start_rdma  ) (); /* Init. transfert DMA depuis disk    */
    IWORD          (*cancel_dma  ) (); /* Annuler transfert DMA en cours     */
    IWORD          (*get_bus     ) (); /* Prendre la ressource Bus           */
    IWORD          (*select_bus  ) (); /* Modifier position sur le bus       */
    IWORD          (*let_bus     ) (); /* Relacher la ressource Bus          */
    IWORD          (*end         ) (); /* Fin d'utilisation du controleur    */
  END               Ata_desc         ; /* ---------------------------------- */


/*
    2.6 Contexte et description de FAN
    ----------------------------------
*/


#define NBREG_FAN      4               /* Nombre de registres driver FAN     */

typedef struct _Fan_ctx                /* Descr. d'un contexte FAN           */
  BEGIN                                /* ---------------------------------- */
    IWORD           (*set      )  () ; /* Ad proc modif alim ventilateur     */
    INT             reg[NBREG_FAN]   ; /* Tableau des registres              */
    IWORD           last_temp        ; /* Temperature precedente             */
    UINT            iod              ; /* Iod de la voie logique             */
    IWORD           idto             ; /* Identifiant tempo courante         */
    IWORD           cycle            ; /* Duree d'un cycle PWM (en ms)       */
    IWORD           tim_on           ; /* Duree etat haut du PWM             */
    IWORD           tim_off          ; /* Duree etat bas du PWM              */
    IWORD           cpt_cycles       ; /* Compteur de cycles PWM             */
    IWORD           soft_pwm         ; /* Dit si PWM soft(1) ou hard(0)      */
    IWORD           is_hdd           ; /* Dit si controleur disque present   */
    IWORD           hdd_dead         ; /* Dit si un disque dur est present   */
    IWORD           wsmart           ; /* Indique si attente RESP_SMART      */
    IWORD           nxtsmart         ; /* Compteur duree entre REQ_SMART     */
    IWORD           def_speed        ; /* Vitesse par defaut du ventilo      */
    IWORD           min_speed        ; /* Vitesse minimum du ventilo         */
    IWORD           max_speed        ; /* Vitesse maximum du ventilo         */
    IWORD           ini_algo         ; /* Indique etape algo regul temp      */
    IWORD           tab_temp[5]      ; /* Dernieres temp. mesurees           */
    IWORD           idx_temp         ; /* Indice ecr. dans "tab_temp[]"      */
    IWORD           flags            ; /* Flags des evenements survenus      */
    USHORT          reqemet          ; /* Num aut emetteur REQ_OPEN/CLOSE    */
    USHORT          reqevl           ; /* VL emettrice REQ_OPEN/REQ_CLOSE    */
    USHORT          reqid            ; /*Id utilisateur REQ_OPEN/REQ_CLOSE   */
    USHORT          sessid           ; /* Numero de session                  */
  END               Fan_ctx          ; /* ---------------------------------- */

typedef struct _Fan_desc               /* Descr. driver bas niveau FAN       */
  BEGIN                                /* ---------------------------------- */
    IWORD           (*info     )  () ; /* Ad proc recup infos statiques      */
    IWORD           (*set      )  () ; /* Ad proc modif alim ventilateur     */
  END               Fan_desc         ; /* ---------------------------------- */


/*
    2.7  Contexte et description de NAND
    ------------------------------------
*/


typedef struct                         /* Descr. driver bas niveau NAND      */
  BEGIN                                /* ---------------------------------- */
    UINT            type             ; /* Type de flash. 0:STA 1:STB etc.    */
    UINT            nbblk            ; /* Nb total de blocs dans la flash    */
    UINT            secperblk        ; /* Nb de secteurs par bloc            */
    UINT            tdata            ; /* Nb octets de donnees par secteur   */
    UINT            tspare           ; /* Nb octets de spare   par secteur   */
  END               Nand_def         ; /* ---------------------------------- */

typedef struct                         /* Descr. driver bas niveau NAND      */
  BEGIN                                /* ---------------------------------- */
    IWORD           (*init     )  () ; /* Initialiser l'acces a la flash     */
    IWORD           (*end      )  () ; /* Fin d'utilisation de la flash      */
    IWORD           (*start    )  () ; /* Demarrer une transaction           */
    IWORD           (*stop     )  () ; /* Terminer une transaction           */
    IWORD           (*wait     )  () ; /* Attendre RBN                       */
    IWORD           (*cmd      )  () ; /* Ecrire des octets de commande      */
    IWORD           (*adr      )  () ; /* Ecrire des octets d'adresse        */
    IWORD           (*write    )  () ; /* Ecrire des octets de donnees       */
    IWORD           (*read     )  () ; /* Lire des octets de donnees         */
  END               Nand_desc        ; /* ---------------------------------- */


/*
    2.8  Contexte et description de ETHERNET/ARP/FAKE
    -------------------------------------------------
*/


typedef struct                         /* Structure decrivant un node        */
  BEGIN                                /* ---------------------------------- */
    USHORT          mac[3]           ; /* Adresse MAC du node distant        */
                                       /* ---------------------------------- */
    UCHAR          *rxbuffer         ; /* Buffer en cours de reception       */
    UCHAR          *txbuffer         ; /* Buffer eb cours de transmission    */
    UCHAR          *rxchain          ; /* Chaine de buffers en reception     */
    UCHAR          *txchain          ; /* Chaine de buffers en transmission  */
    UINT            mtu              ; /* Transmission protocol MTU          */
    UINT            rxindex          ; /* Offset de reception dans le buffer */
    UINT            txindex          ; /* Offset de transmission dans buffer */
    INT             rxoff            ; /* Offset des donnees dans le buffer  */
    INT             txoff            ; /* Offset des donnees dans le buffer  */
    INT             rxsize           ; /* Taille des donnees dans le buffer  */
    INT             txsize           ; /* Taille des donnees dans le buffer  */
    UCHAR           rxcpt            ; /* Prochain compteur attendu reception*/
    UCHAR           txcpt            ; /* Prochain compteur a emettre        */
    UCHAR           index            ; /* Index du node dans le tableau node */
    UCHAR           status           ; /* Statut du node                     */
    S_evt           rxevt            ; /* Evenement en cours de reception    */
    S_evt           txevt            ; /* Evenement en cours de transmission */
                                       /* ---------------------------------- */
    UINT            statrxpkt        ; /* Nombre trames recues de ce mode    */
    UINT            statrxevt        ; /* Nombre trames recues de ce mode    */
    UINT            statrxdrop       ; /* Nombre de trames droppees en rx    */
    UINT            statrxinval      ; /* Nombre de trames invalides en rx   */
    UINT            statrxlost       ; /* Nombre de trames perdues en rx     */
    UINT            stattxpkt        ; /* Nombre trames emises vers ce node  */
    UINT            stattxevt        ; /* Nombre trames emises vers ce node  */
    UINT            stattxdrop       ; /* Nombre de trames droppees en tx    */
    UINT            statallocerror   ; /* Nombre de trames droppees en tx    */
    UINT            statprotoerror   ; /* Nombre de trames droppees en tx    */
  END               Hl_node          ; /* ---------------------------------- */

typedef struct _Eth_ctx                /* Structure decrivant un contexte ETH*/
  BEGIN                                /* ---------------------------------- */
    INT            iod               ; /* IOD associe au device ouvert       */
    USHORT         open_emet         ; /* Champ "reqemet" du REQ_OPEN        */
    USHORT         open_evl          ; /* Champ "reqevl"  du REQ_OPEN        */
    USHORT         open_node         ; /* Champ "reqnode" du REQ_OPEN        */
    USHORT         open_id           ; /* Champ "reqid"   du REQ_OPEN        */
    USHORT         open_sid          ; /* Champ "sessid"  du REQ_OPEN        */
    IWORD          (*getsta  )()     ; /* Obtention de l'etat du cable reseau*/
    IWORD          (*start   )()     ; /* Demarrage de l'interface reseau    */
    IWORD          (*stop    )()     ; /* Arret     de l'interface reseau    */
    IWORD          (*receive )()     ; /* Ajout d'un buffer de reception     */
    IWORD          (*send    )()     ; /* Emission d'une trame               */
    UINT           idto_phy          ; /* Identifiant du timer polling PHY   */
    IWORD          physta            ; /* Etat du cable Ethernet selon le PHY*/
    USHORT         pro_emet          ; /* Champ "reqemet" du REQ_IFBIND      */
    USHORT         pro_evl           ; /* Champ "reqevl"  du REQ_IFBIND      */
    USHORT         pro_node          ; /* Champ "reqnode" du REQ_IFBIND      */
    UCHAR          mac_adr[6]        ; /* Adresse MAC                        */
    UCHAR          pro_adr[SZ_PRO_ADR];/* Adresse protocolaire               */
    UCHAR          pro_len           ; /* Nb octets dans l'adresse protocol. */
    UCHAR          txnode            ; /* Tunnelling vers ce node en cours   */
    USHORT         pro_typ           ; /* Type Ethernet protocole niveau sup */
    USHORT         lowdev            ; /* lower device index                 */
                                       /* ---------------------------------- */
    IWORD          started           ; /* 1 si le MAC est demarre.           */
    IWORD          lack              ; /* Nb de buffers dont on a rate alloc.*/
    IWORD          ipc               ; /* IP Checksum capabilities           */
                                       /* ---------------------------------- */
    IWORD          lec               ; /*                                    */
    IWORD          ecr               ; /*                                    */
    IWORD          nb                ; /*                                    */
    UCHAR         *frm_ad[ETH_NBRX]  ; /*                                    */
    INT            frm_lg[ETH_NBRX]  ; /*                                    */
    UINT           frm_ts[ETH_NBRX]  ; /*                                    */
                                       /* ---------------------------------- */
    Hl_node        node[ETH_NBNODE]  ; /* Tableau contenant les nodes connus */
    UINT           rxdrop            ; /* Statistique des trames jetees en   */
    UINT           txdrop            ; /* mode tunnel                        */
                                       /* ---------------------------------- */
    IWORD          bsy               ; /* Flag trt d'un REQ_WRITE en cours   */
    IWORD          nbwait            ; /* Nb de paquets en attente d'emission*/
    USHORT         wri_ecr           ; /*                                    */
    USHORT         wri_lec           ; /*                                    */
    S_evt          wri_evt[ETH_NBTX] ; /*                                    */
                                       /* ---------------------------------- */
    USHORT         sessid            ; /* Id. comm entre AUT_ETH et AUT_ARP. */
                                       /* ---------------------------------- */
    S_evt          delayq[ETH_DELAYQ]; /*                                    */
    INT            delay_lec         ; /*                                    */
    INT            delay_ecr         ; /*                                    */
    INT            delay_nb          ; /*                                    */
    INT            delay_tid         ; /* Timer Id                           */
    INT            delay_us          ; /* Cumulative delay                   */
  END              Eth_ctx           ; /* ---------------------------------- */

typedef struct                         /* Desc driver bas niveau ETHERNET    */
  BEGIN                                /* ---------------------------------- */
    IWORD           (*init    )()    ; /* Initialision                       */
    IWORD           (*getsta  )()    ; /* Obtenir l'etat du cable reseau     */
    IWORD           (*start   )()    ; /* Demarrer l'interface               */
    IWORD           (*stop    )()    ; /* Arreter  l'interface               */
    IWORD           (*receive )()    ; /* Ajout d'un buffer de reception     */
    IWORD           (*send    )()    ; /* Emission d'une trame               */
  END               Eth_desc         ; /* ---------------------------------- */

typedef struct _Arp_entry              /* Structure entree du cache ARP      */
  BEGIN                                /* ---------------------------------- */
    UCHAR           pro[SZ_PRO_ADR]  ; /* Adresse protocolaire               */
    UCHAR           mac[6]           ; /* Adresse MAC                        */
    INT             to_remove        ; /* Time-out avant suppression entree  */
    IWORD           used             ; /* Drapeau d'allocation.              */
  END               Arp_entry        ; /* ---------------------------------- */

typedef struct _Arp_job                /* Structure d'un job ARP             */
  BEGIN                                /* ---------------------------------- */
    S_evt           evtreq           ; /* Copie de l'evt REQ_RESOLVE         */
    UCHAR           pro_adr[SZ_PRO_ADR]; /* Adresse protocolaire recherchee. */
    USHORT          nbretry          ; /* Nb de tentatives restantes         */
    USHORT          used             ; /* Drapeau d'allocation.              */
    INT             to_retry         ; /* Time-out avant prochaine tentative */
  END               Arp_job          ; /* ---------------------------------- */

typedef struct _Arp_ctx                /* Structure decrivant un contexte ARP*/
  BEGIN                                /* ---------------------------------- */
    INT             iod              ; /* IOD associe au device Ethernet.    */
    INT             sessid           ; /* Champ sessid commun avec ETH.      */
    INT             mode             ; /* Mode de defense de l'adresse IP.   */
    INT             defense          ; /* Date de derniere defense.          */
    UCHAR           mac_adr[6]       ; /* Adresse MAC                        */
    UCHAR           pro_adr[SZ_PRO_ADR]; /* Adresse protocolaire             */
    UCHAR           pro_len          ; /* Nb octets dans l'adresse protocol. */
    USHORT          pro_typ          ; /* Type Ethernet protocole niveau sup */
    UINT            idto             ; /* Identifiant de timeout generique.  */
    UCHAR           num              ; /* Compteur generique (PROBE/ANNOUNCE)*/
    UINT            idto_retry       ; /* Identifiant de timeout RETRY.      */
    Arp_entry       cache[ARP_SZCACHE] ; /* Table correspondance pro/phy     */
    Arp_job         tjob[ARP_NBJOB]  ; /* Table des jobs ARP.                */
  END               Arp_ctx          ; /* ---------------------------------- */

typedef struct                         /* Pro_desc module compagnon PRO_FAKE */
  BEGIN                                /* ---------------------------------- */
    IWORD           (*init    )()    ; /* Traitement de REQ_INIT             */
    IWORD           (*end     )()    ; /* Traitement de REQ_END              */
    IWORD           (*alloc   )()    ; /* Demande allocation de contexte     */
    IWORD           (*free    )()    ; /* Demande liberation de contexte     */
    IWORD           (*open    )()    ; /* Traitement de REQ_OPEN             */
    IWORD           (*close   )()    ; /* Traitement de REQ_CLOSE            */
    IWORD           (*allocsub)()    ; /* Demande allocation sous-contexte   */
    IWORD           (*freesub )()    ; /* Demande liberation sous-contexte   */
    IWORD           (*connect )()    ; /* Traitement de REQ_CONNECT          */
    IWORD           (*listen  )()    ; /* Traitement de REQ_LISTEN           */
    IWORD           (*breakc  )()    ; /* Traitement de IND_DISCONNECT       */
    IWORD           (*disc    )()    ; /* Traitement de REQ_DISCONNECT       */
    IWORD           (*abort   )()    ; /* Traitement de REQ_ABORT            */
    IWORD           (*start   )()    ; /* Traitement de REQ_START            */
    IWORD           (*stop    )()    ; /* Traitement de REQ_STOP             */
    IWORD           (*write   )()    ; /* Traitement de REQ_WRITE            */
    IWORD           (*read    )()    ; /* Traitement de REQ_READ             */
    IWORD           (*setval  )()    ; /* Traitement de REQ_SETVAL           */
    IWORD           (*getval  )()    ; /* Traitement de REQ_GETVAL           */
    IWORD           (*tx      )()    ; /* Traitement de IT_TX                */
    IWORD           (*rx      )()    ; /* Traitement de IT_RX                */
    IWORD           (*to      )()    ; /* Traitement de TO_...               */
    IWORD           (*sta     )()    ; /* Demande lecture etat               */
  END               Fake_desc        ; /* ---------------------------------- */

typedef struct                         /* Dev_def module compagnon FAKE      */
  BEGIN                                /* ---------------------------------- */
    CHAR           *name             ; /* Adresse du nom                     */
    UCHAR           lowdev           ; /* lower device index                 */
    UCHAR           reqmask          ; /* Indique les REQ_READ/IFBIND/IFSTART*/
    UINT            szbufread        ; /* Taille buf alloues par le driver   */
    IWORD           nbtokopen        ; /* Nb token apres le REQ_OPEN         */
    IWORD           rdszout          ; /* FIFO READ  - Taille zone "out"     */
    IWORD           rdszin           ; /* FIFO READ  - Taille zone "in"      */
    IWORD           wrszout          ; /* FIFO WRITE - Taille zone "out"     */
    IWORD           wrszin           ; /* FIFO WRITE - Taille zone "in"      */
    UCHAR           tag_state        ; /* Numero du tag (TAG_ETH_STATE)      */
  END               Fake_def         ; /* ---------------------------------- */

typedef struct                         /* File d'evenements de requete       */
  BEGIN                                /* ---------------------------------- */
    S_evt           *fifo            ; /* Ad de debut de la file circulaire  */
    IWORD           szout            ; /* Taille de la zone "out"            */
    IWORD           szin             ; /* Taille de la zone "in"             */
    IWORD           pout             ; /* Prochain a sortir de "out          */
    IWORD           pinout           ; /* Prochain a passer de "in" a "out"  */
    IWORD           pin              ; /* Prochain a entrer dans "in"        */
    IWORD           nout             ; /* Nombre dans "out"                  */
    IWORD           nin              ; /* Nombre dans "in"                   */
  END               Fifo_evt         ; /* ---------------------------------- */

typedef struct _Fake_ctx               /* Debut de contexte FAKE             */
  BEGIN                                /* ---------------------------------- */
    Fake_desc      *desc             ; /* Ad "Fake_desc" du device           */
    Fake_def       *def              ; /* Ad "Fake_def"                      */
    USHORT          dev              ; /* Numero de device                   */
    USHORT          aut              ; /* Numero de l'automate               */
                                       /* ---------------------------------- */
    UINT            iod              ; /* IOD de ce device                   */
    Resp_info       open             ; /* Infos du REQ_OPEN                  */
    Resp_info       bind             ; /* Infos du REQ_IFBIND                */
    UINT            openflags        ; /* Champ "flags du REQ_OPEN           */
                                       /* ---------------------------------- */
    Fifo_evt        write            ; /* REQ_WRITE en attente               */
    Fifo_evt        read             ; /* REQ_READ  en attente               */
                                       /*                                    */
    IWORD           lowsta           ; /* Etat du dessous                    */
    UINT            idto             ; /* Identifiant temporisateur TO_PHY   */
                                       /* ---------------------------------- */
    UCHAR           mac_adr[8]       ; /* Adresse MAC                        */
    UCHAR           mac_len          ; /* Nb octets de l'adresses MAC        */
    UCHAR           pro_adr[SZ_PRO_ADR];/* Adresse protocolaire              */
    UCHAR           pro_len          ; /* Nb octets dans l'adresse protocol. */
    USHORT          pro_typ          ; /* Type Ethernet protocole niveau sup */
                                       /* ---------------------------------- */
    IWORD           ipc              ; /* IP Checksum capabilities           */
  END               Fake_ctx         ; /* ---------------------------------- */
 

/*
    2.9 Contexte et description de PLAY
    -----------------------------------
*/


typedef struct _Url_prs                /* Structure contexte parsing URL.    */
  BEGIN                                /* ---------------------------------- */
    UCHAR       *buffer              ; /* Adresse chaine de caracteres URL.  */
    IWORD        etat                ; /* Etat de l'automate de parsing.     */
    IWORD        postok              ; /* Indice 1er caractere du token.     */
    IWORD        lgtok               ; /* Longueur du token.                 */
    IWORD        i                   ; /* Indice courant dans la chaine.     */
  END            Url_prs             ; /* ---------------------------------- */

typedef struct _Http_prs               /* Structure ctx parsing reponse HTTP.*/
  BEGIN                                /* ---------------------------------- */
    IWORD        type                ; /* 0:HTTP    1:RTSP                   */
    IWORD        etat                ; /* Etat de parsing de la reponse HTTP.*/
    IWORD        ss_etat             ; /* Sous-etat de parsing reponse HTTP. */
    UINT         i                   ; /* Indice d'ecriture dans buffer.     */
    UCHAR        keptchar            ; /* Cara. conserve pour trt ulterieur. */
    IWORD        major               ; /* Numero de version majeur HTTP.     */
    IWORD        minor               ; /* Numero de version mineur HTTP.     */
    IWORD        status              ; /* Code de status de la reponse HTTP  */
    CHAR         phrase[256]         ; /* Copie contenu phrase de status.    */
    CHAR         field [1024]        ; /* Copie contenu champ d'en-tete.     */
    CHAR         value [1024]        ; /* Copie contenu valeur du champ.     */
    IWORD        entity              ; /* Structure de l'entite HTTP.        */
    UINT         length              ; /* Taille chunk ou entite (si connue).*/
  END            Http_prs            ; /* ---------------------------------- */

typedef struct _Play_ctx               /* Structure contexte canal Play.     */
  BEGIN                                /* ---------------------------------- */
    IWORD           used             ; /* 1 si contexte alloue, 0 si libre.  */
    INT             iod              ; /* IOD du contexte de connexion Play. */
    INT             memuid           ; /* Id utilisateur memoire             */
    IWORD           scheme           ; /* "Scheme" choisit pour le contexte. */
    UCHAR          *adresse          ; /* Champ "adresse" de la requete.     */
    IWORD           coderesp         ; /* Champ "code" de la reponse.        */
    USHORT          cnx_emet         ; /* Num aut emetteur REQ_CONNECT       */
    USHORT          cnx_evl          ; /* VL emettrice req REQ_CONNECT       */
    USHORT          cnx_id           ; /* Id. utilisateur req REQ_CONNECT    */
    USHORT          cnx_sid          ; /* Numero de session REQ_CONNECT      */
    USHORT          dcnx_emet        ; /* Num aut emetteur REQ_DISCONNECT    */
    USHORT          dcnx_evl         ; /* VL emettrice req REQ_DISCONNECT    */
    USHORT          dcnx_id          ; /* Id. utilisateur req REQ_DISCONNECT */
    USHORT          dcnx_sid         ; /* Numero de session REQ_DISCONNECT   */
    USHORT          reqemet          ; /* Num aut emetteur req READ/WRITE    */
    USHORT          reqevl           ; /* VL emettrice req READ/WRITE        */
    USHORT          reqid            ; /* Id. utilisateur requete READ/WRITE */
    USHORT          sessid           ; /* Numero de session                  */
    IWORD           state            ; /* Etat protocolaire courant.         */
    IWORD           numtra           ; /* Numero transition en cours.        */
    E32_ctx         e32              ; /* Contexte d'execution du prog EMU32.*/
    INT             stack[64]        ; /* Pile d'execution du prog EMU32.    */
    UINT            addr1            ; /* Valeur du tag ADDR1.               */
    UINT            addr2            ; /* Valeur du tag ADDR2.               */

/* Variables utilisees par le scheme "file" ================================ */

    CHAR            filepath[256]    ; /* Chemin d'acces au fichier local.   */

/* Variables utilisees par le scheme "http" ================================ */

    UCHAR          *lnkhdr           ; /* Adr buf en-tete chaine bufs en-tete*/
    UCHAR          *bufcur           ; /* Adr buf courant pour champs en-tete*/
    UINT            of7cur           ; /* Of7 buf courant pour champs en-tete*/
    Http_prs        http_prs         ; /* Contexte de parsing reponse HTTP.  */
    UCHAR          *http_buf         ; /* Adresse buffer rendu par auto TCP  */
    UINT            http_of7         ; /* Offset donnees utiles dans buf     */
    UINT            http_sz          ; /* Taille donnees utiles dans buf     */

/* Variables utilisees par le scheme "rtsp" ================================ */

    CHAR            session[64]      ; /* Copie du dernier champ "session".  */
    IWORD           more             ; /* A VIRER QUAND pipe.c   SERA FINI   */
  END               Play_ctx         ; /* ---------------------------------- */


/*
    2.10 Contexte et description de GLUE
    ------------------------------------
*/


typedef struct _Gluevol_ctx            /* Structure decrivant volume GLUE    */
  BEGIN                                /* ---------------------------------- */
    IWORD           used             ; /* 0:volume monte   1: volume demonte */
    IWORD           iod              ; /* IOD du volume                      */
    IWORD           iod_fil          ; /* IOD de base pour les fichiers      */
  END               Gluevol_ctx      ; /* ---------------------------------- */

typedef struct _Gluefil_ctx            /* Structure decrivant fichier GLUE   */
  BEGIN                                /* ---------------------------------- */
    IWORD           used             ; /* 0:ctx libre   1:ctx utilise        */
    CHAR            path[MAX_DISKPATH];/* Nom du fichier (chemin absolu)     */
    IWORD           opened           ; /* b0:ouvert lecture  b1:ecriture     */
    UCHAR          *link             ; /* Tete de chaine du tampon GLUE      */
    UINT            pos              ; /* Position lecture courante tampon   */
    UCHAR          *buf              ; /* Adresse buffer contenant "pos"     */
    INT             of7              ; /* Offset de "pos" dans buffer "buf"  */
    INT             lg               ; /* Nb octets apres "pos" dans "buf"   */
    UINT            sz               ; /* Compte octets deja lus du tampon.  */
    UINT            max              ; /* Taille maximum autorisee du tampon */
    UINT            seek             ; /* Position demandee par REQ_GSEEK    */
    IWORD           vl               ; /* Numero vl du canal GLUE associe    */
    IWORD           eof              ; /* Drapeau fin de fichier atteinte    */
  END               Gluefil_ctx      ; /* ---------------------------------- */

typedef struct _Gluesrv_ctx            /* Structure decrivant canal GLUE     */
  BEGIN                                /* ---------------------------------- */

/* Donnees generiques d'un canal GLUE .......................................*/
    IWORD           used             ; /* 0:ctx libre   1:ctx utilise        */
    USHORT          vl               ; /* Numero de vl de ce canal GLUE.     */
    INT             iod              ; /* IOD du canal GLUE.                 */
    USHORT          reqemet          ; /* reqemet du REQ_OPEN ou du REQ_CLOSE*/
    USHORT          reqevl           ; /* reqevl du REQ_OPEN ou du REQ_CLOSE */
    USHORT          reqnode          ; /* reqnode du REQ_OPEN ou du REQ_CLOSE*/
    USHORT          reqid            ; /* reqid du REQ_OPEN ou du REQ_CLOSE. */
    USHORT          sessid           ; /* sessid du REQ_OPEN ou du REQ_CLOSE.*/
    INT             reg[GLUE_NBREG]  ; /* Registres d'interface du module.   */
    INT             error            ; /* Code derniere erreur ayant eu lieu.*/
    INT             iod_play         ; /* IOD du canal PLAY.                 */
    USHORT          aut_play         ; /* Numero de l'automate AUT_PLAY.     */
    USHORT          vl_play          ; /* Numero de la voie logique AUT_PLAY.*/

/* Donnees utilisees en mode "push" .........................................*/
    UCHAR          *blk_buf          ; /* Buffer bloc du fichier cible.      */
    INT             blk_of7          ; /* Offset courant dans buffer bloc.   */
    INT             blk_sz           ; /* Taille totale du buffer bloc.      */
    UCHAR          *tcp_buf          ; /* Dernier buffer TCP recu.           */
    INT             iod_disk         ; /* IOD du fichier cible.              */
    USHORT          aut_disk         ; /* Num auto systeme de fichiers cible.*/
    USHORT          vl_disk          ; /* Num vl systeme de fichiers cible.  */
    UCHAR           maj_disk         ; /* Major number systeme de fichiers.  */

/* Donnees utilisees en mode "pull" .........................................*/
    IWORD           numfil           ; /* Numero du fichier virtuel associe. */
    IWORD           full             ; /* 1 si ecriture gelee parce que plein*/
    IWORD           mapped           ; /* Portion d'un fichier a conserver   */
    UINT            mapof7           ; /* Of7 1er octet portion fichier cons.*/
    IWORD           wresp            ; /* Drapeaux d'attentes de reponses    */
    IWORD           seeking          ; /* 1 si demande de seek en cours      */
    UINT            seekid           ; /* Id. de la requete de positionnement*/
    INT             to_pgseek        ; /* Id. tempo repetition REQ_GSEEK     */
    INT             to_pfget         ; /* Id. tempo repetition REQ_FGET      */
  END               Gluesrv_ctx      ; /* ---------------------------------- */

typedef struct glue_desc                     /* Interface avec un "glue"     */
  BEGIN                                      /*                              */
    IWORD (*connect_src )(INT, INT, IWORD,   /* Connexion en tant que source */
                          UCHAR** , INT*  ); /*                              */
    IWORD (*put         )(INT, INT        ); /* Initier ou reprendre les     */
                                             /* transferts depuis la source  */
    IWORD (*seek        )(INT, INT, UINT  ); /* Positionnement fichier       */
    void  (*set_rptr    )(INT, INT, UCHAR*); /* MAJ du pointeur de lecture   */
    IWORD (*connect_sink)(INT, INT, IWORD,   /* Connexion en tant que puit   */
                          UCHAR*  , INT  ,   /*                              */
                          UCHAR** , INT*  ); /*                              */
    IWORD (*get         )(INT, INT        ); /* Initier ou reprendre les     */
                                             /* transferts vers le puit      */
    void  (*disconnect  )(INT, INT        ); /* Deconnexion                  */
  END  Glue_desc                           ; /*                              */


/*
    2.11 Contexte et description de HTTP
    ------------------------------------
*/


typedef struct _Http_ctx               /* Structure decrivant un ctx HTTP    */
  BEGIN                                /* ---------------------------------- */
    IWORD           used             ; /* 0:ctx libre   1:ctx utilise        */
    INT             iod              ; /* IOD du canal HTTP.                 */
    INT             memuid           ; /* Id utilisateur memoire             */
    IWORD           numctx           ; /* Numero du contexte HTTP courant.   */
    IWORD           tcp_vl           ; /* Numero VL automate AUT_IP          */
    USHORT          cnx_emet         ; /* Copie "reqemet" du REQ_CONNECT     */
    USHORT          cnx_evl          ; /* Copie "reqevl" du REQ_CONNECT      */
    USHORT          cnx_id           ; /* Copie "reqid" du REQ_CONNECT       */
    USHORT          cnx_sid          ; /* Copie "sessid" du REQ_CONNECT      */
    USHORT          dcnx_emet        ; /* Copie "reqemet" du REQ_DISCONNECT  */
    USHORT          dcnx_evl         ; /* Copie "reqevl" du REQ_DISCONNECT   */
    USHORT          dcnx_id          ; /* Copie "reqid" du REQ_DISCONNECT    */
    USHORT          dcnx_sid         ; /* Copie "sessid" du REQ_DISCONNECT   */
    UINT            itf [NBREG_HTTP] ; /* Table des registres d'interface.   */
    UINT            itf2[NBREG_HTTP] ; /* Table des registres d'interface 2. */
    UCHAR          *urlapp           ; /* Buf telecom URL applicative.       */
    UCHAR          *urlsav           ; /* Buf telecom URL sauvee au connect. */
    IWORD           etat             ; /* Etat de l'automate HTTP interne.   */
    IWORD           connected        ; /* 1 si un pipe est connecte          */
    IWORD           openedin         ; /* 0:robinet ferme  1:robinet ouvert  */
    IWORD           openedout        ; /* 0:robinet ferme  1:robinet ouvert  */
    IWORD           pgmsta           ; /* Etat du programme interprete       */
    IWORD           pgmerr           ; /* Code d'err. en sortie du prog inter*/
    IWORD           cancel           ; /* 1 si le programme doit etre annule.*/
    IWORD           newpipe          ; /* Vrai si utilisation NEW PIPE       */
    INT             pipeidin         ; /* Id. pipe connecte a une "source".  */
    INT             pipeidout        ; /* Id. pipe connecte a un "puits".    */
    IWORD           seeking          ; /* 1 si demande de seek en cours.     */
    UINT            seekof7          ; /* Position de demarrage apres seek.  */
    IWORD           int_permitted    ; /* 1 si un cancel peut reveiller prog.*/
    INT             evtrel           ; /* Numero de l'evenement attendu.     */
    INT             regret           ; /* Num reg EMU32 pour code de retour. */
    INT             regpar[2]        ; /* Num reg EMU32 pour params de retour*/
                                       /* ---------------------------------- */
    E32_ctx         e32              ; /* Contexte d'execution du prog EMU32 */
    INT             stack[64]        ; /* Pile d'execution du prog EMU32     */
                                       /* ---------------------------------- */
    Http_prs        http_prs         ; /* Contexte de parsing reponse HTTP.  */
                                       /* ---------------------------------- */
    IWORD           glue_killed      ; /* A VIRER QUAND pro_ip.c SERA FINI   */
  END               Http_ctx         ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


#endif                                 /* End of INCLUDE_H.                  */
