/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/req_lib.h              */
/*  Constants and definitions of request library interface.                  */
/*                                                                           */
/*  Author : JYA                        Last modification : 09/21/2020       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines

         2  Structures and typedefs
            2.1  One context table description structure - Tab_ctx
            2.2  Context header structure - Hd_ctx
            2.3  Channel context - Req_ctx/Chk_fifo4
            2.4  Sub-channel context - Req_subctx
            2.5  Job request - Job et Job_data
            2.6  Contexte automate de dialogue - Dia_ctx
            2.7  Une clef et ses 4 tetes de chaines - Key_lnk
            2.8  Tete de chaine de jobs - Hd_joblnk/Hd_jobfree
            2.9  Structure de lien avec la couche basse ou a droite
            2.10 Structure buffer circulaire

         3  Public variables

         4  Prototypes of public procedures


*/


#ifndef REQ_LIB_H                      /* If this file is not already        */
#define REQ_LIB_H                      /* included, include it.              */


/*
    1    Defines
    ____________
*/


/*
    2    Structures and typedefs
    ____________________________
*/


/*
   2.1  One context table description structure - Tab_ctx
   ------------------------------------------------------
*/

typedef struct     tctx                /* One context table description      */
  BEGIN                                /* ---------------------------------- */
    struct hctx  **ctxptr            ; /* Adr Context array table adress     */
    UINT           szctxptr          ; /* Size of adress array               */
    struct hctx   *stactx            ; /* Adr static context array           */
    UINT           szstactx          ; /* Size of context array              */
    UINT           ctxlen            ; /* Size in byte of a context          */
    UINT           ctxnball          ; /* Dynamic allocation grain           */
    UINT           flags             ; /* Flags                              */
                                       /* ---------------------------------- */
    UINT           of7rctx           ; /* Offset in context for  Req_ctx     */
    UINT           of7keylnk         ; /* Offset in context for  Key_lnk     */
    UINT           keylen            ; /* Size in byte of a key              */
                                       /* ---------------------------------- */
    UINT           nbbuf             ; /* Number of allocated buffers        */
    UINT           nbctx             ; /* Number of allocated contexts       */
    struct hctx   *first             ; /* Adress of first allocated buffer   */
  END              Tab_ctx           ; /* ---------------------------------- */


/*
   2.2  Context header structure - Hd_ctx
   --------------------------------------
*/

typedef struct   hctx                  /* En-tete d'un contexte              */
  BEGIN                                /* ---------------------------------- */
    struct tctx   *tabctx            ; /* Adresse de la table de contextes   */
    USHORT         ictx              ; /* Indice/Identifiant de ce contexte  */
    USHORT         nbref             ; /* Compteur de references             */
    struct hctx   *next              ; /* Adresse contexte alloue suivant    */
  END              Hd_ctx            ; /* ---------------------------------- */

/*
   2.3  Description d'un contexte de "req_lib.c" - Req_ctx/Chk_req_ctx
   -------------------------------------------------------------------
*/

typedef struct                         /* Files req/ecr/lec/aut et compteur  */
  BEGIN                                /* ---------------------------------- */
    UINT           iodsys            ; /* IOD systeme   de ce contexte       */
    USHORT         myaut             ; /* Numero automate VMIO associe       */
    USHORT         myvl              ; /* Numero Voie Logique associee       */
    USHORT         mydrvsys          ; /* Numero VMIO de driver SYSTEME      */
                                       /*                                    */
    UINT           iodcar            ; /* IOD VMOS    de ce contexte         */
    UINT           flags             ; /* Champ "flags" du REQ_ALLOC         */
    USHORT         mydrvcar          ; /* Numero VMIO de driver CARACTERE    */
    Resp_info      resp_info         ; /* reqemet/reqevl/reqnode/reqid/sessid*/
                                       /*                                    */
    IWORD        (*trt_job)()        ; /* Ad procedure trt specifique job    */
    IWORD        (*evt_end)()        ; /* Ad procedure evenement de fin      */
                                       /*                                    */
    void           *useradr          ; /* User defined address               */
  END              Req_ctx           ; /* ---------------------------------- */


typedef struct                         /* Param pour procedure "chk_req_ctx" */
  BEGIN                                /* ---------------------------------- */
    USHORT         aut               ; /* Valeur attendue du numero d'auto   */
    USHORT         vlmin             ; /* Plus petite valeur de "myvl"       */
    USHORT         vlmax             ; /* Plus grande valeur de "myvl"       */
  END              Chk_req_ctx       ; /* ---------------------------------- */


/*
   2.4  Sub-channel context - Req_subctx
   -------------------------------------
*/

typedef struct                         /* Files req/ecr/lec/aut et compteur  */
  BEGIN                                /* ---------------------------------- */
    USHORT         myaut             ; /* associated VMIO FSM number         */
    USHORT         myvl              ; /* associated Logical Way number      */
                                       /*                                    */
    UINT           iodsub            ; /* sub-channel VMK identifier         */
    USHORT         reqemet           ; /* Numero automate emetteur requete   */
    USHORT         reqevl            ; /* Numero VL       emetteur requete   */
    USHORT         reqnode           ; /* Numero noeud    emetteur requete   */
    USHORT         reqid             ; /* Identifiant de requete             */
    USHORT         sessid            ; /* Identifiant de la session          */
  END              Req_subctx        ; /* ---------------------------------- */


/*
   2.5  Description d'un job - Job et Job_data
   -------------------------------------------
*/

typedef struct    job                  /* Description d'un Job               */
  BEGIN                                /* ---------------------------------- */
    USHORT         myvl              ; /* Associated Logical Way Number      */
                                       /*                                    */
    USHORT         code              ; /* Code de l'operation demandee       */
    USHORT         file              ; /* file d'attente + files bloquantes  */
    UCHAR         *adresse           ; /* Adresse d'emission ou reception    */
    INT            longueur          ; /* Nb d'octets a lire ou ecrire       */
    UINT           flags             ; /* Drapeaux                           */
    UINT           param             ; /* Duree maximale en millisecondes    */
                                       /*                                    */
    USHORT         reqemet           ; /* Numero automate emetteur requete   */
    USHORT         reqevl            ; /* Numero VL       emetteur requete   */
    USHORT         reqnode           ; /* Numero noeud    emetteur requete   */
    USHORT         reqid             ; /* Identifiant de requete             */
    USHORT         sessid            ; /* Identifiant de la session          */
                                       /*                                    */
    struct job    *next              ; /* Element suivant dans la file       */
    struct jobf   *jfree             ; /* Ad chaine d'ou on a ete alloue     */
  END              Job               ; /* ---------------------------------- */
 
typedef struct                         /* Descripteur d'un Job + 64 octets   */
  BEGIN                                /* ---------------------------------- */
    struct job     job               ; /* En-tete de job                     */
    UINT           data[16]          ; /* Donnees utilisateur                */
  END              Job_data          ; /* ---------------------------------- */
 

/* Valeurs du champ "code" ..................................................*/

#define O_CONNECT               0      /* Demande de connexion sortante      */
#define O_LISTEN                1      /* Attente d econnexion entrante      */
#define O_DISCONNECT            2      /* Demande de deconnexion             */
#define O_ABORT                 3      /* Demande de terminaison brutale     */
#define O_START_SINK            4      /* Indication demarrage flot donnees  */
#define O_START_SOURCE          5      /* Demande demarrage flot de donnees  */
#define O_STOP                  6      /* Demande d'arret flot de donnees    */
#define O_SND                   7      /* Emission de donnees TXD            */
#define O_RCV                   8      /* Reception de donnees RXD           */
#define O_SETVAL                9      /* Requete REQ_SETVAL                 */
#define O_GETVAL               10      /* Requete REQ_GETVAL                 */
#define O_RPC                  11      /* Requete REQ_RPC                    */
#define O_EVTCON               12      /* Remonter RESP_CONNECT              */
#define O_EVTLISTEN            13      /* Remonter RESP_LISTEN               */
#define O_EVTDISC              14      /* Remonter RESP_DISC                 */
#define O_EVTABORT             15      /* Remonter RESP_ABORT                */
#define O_EVTSTART             16      /* Remonter RESP_START                */
#define O_EVTSTOP              17      /* Remonter RESP_STOP                 */
#define O_EVTWRITE             18      /* Remonter RESP_WRITE                */
#define O_EVTREAD              19      /* Remonter RESP_READ                 */
#define O_EVTSETV              20      /* Remonter RESP_SETVAL               */
#define O_EVTGETV              21      /* Remonter RESP_GETVAL               */
#define O_EVTRPC               22      /* Remonter RESP_RPC                  */
#define O_EVTIOEND             23      /* Remonter EVTS_IO_END - xxx_io      */
#define O_EVTIOERR             24      /* Remonter EVTS_IO_END si erreur     */
#define O_CALLBACK             25      /* Appeler procedure call-back        */
#define O_REQSKIP              26      /* Sauter requetes a suivre si erreur */
#define O_WAITMSEC             27      /* Attente d'une duree en millisec    */
#define O_SNDNEXT              28      /* Emission vers le suivant           */

#define O_READ 40
#define O_WRITE 41

#define B_SYS              0x0100      /* b8  Demander 1:Drv systeme 0;cara  */
#define B_RST              0x0200      /* b9  Demande de reset code erreur   */
#define B_EVT              0x0400      /* b10 En attente d'un evenement      */


/* Bits du champ "file" .....................................................*/

#define W_FILE0           (1<< 4)      /* La file 0 nous bloque              */
#define W_FILE1           (1<< 5)      /* La file 1 nous bloque              */
#define W_FILE2           (1<< 6)      /* La file 2 nous bloque              */
#define W_FILE3           (1<< 7)      /* La file 3 nous bloque              */


/*
   2.6  Contexte automate de dialogue - Dia_ctx
   --------------------------------------------
*/

typedef struct    dia                  /* Contexte automate de dialogue      */
  BEGIN                                /* ---------------------------------- */
    USHORT         myctx             ; /* Num du contexte associee           */
    SHORT          et                ; /* Etat      automate de dialogue     */
    SHORT          ev                ; /* Evenement automate de dialogue     */
    UINT           idto              ; /* Id temporisateur en cours          */
    INT            cnt               ; /* Compteur                           */
  END              Dia_ctx           ; /* ---------------------------------- */


/*
   2.7  Tete de chaine de jobs - Hd_joblnk/Hd_jobfree
   --------------------------------------------------
*/

typedef struct                         /* Tet de Chaine de structures Job    */
  BEGIN                                /* ---------------------------------- */
    USHORT         ilnk              ; /* Indice de cette tete de chaine     */
    struct job    *first             ; /* Ad premier element de la chaine    */
    struct job    *last              ; /* Ad dernier element de la chaine    */
    USHORT         nb                ; /* Nombre d'elements de la chaine     */
    USHORT         skip              ; /* Nbre de requetes a suivre a sauter */
    UINT           idto              ; /* Identifiant de temporisateur       */
  END              Hd_joblnk         ; /* ---------------------------------- */


typedef struct   jobf                  /* Tete de Chaine de Job libres       */
  BEGIN                                /* ---------------------------------- */
    struct job    *first             ; /* Ad premier element de la chaine    */
    USHORT         nb                ; /* Nombre d'elements de la chaine     */
    USHORT         sz                ; /* Taille en octets d'une structure   */
  END              Hd_jfree          ; /* ---------------------------------- */

/*
   2.8  Une clef et ses 4 tetes de chaines - Key_lnk
   -------------------------------------------------
*/
typedef struct                         /* Job structure chain                */
  BEGIN                                /* ---------------------------------- */
    UINT           key[4]            ; /* 16 bytes key value                 */
                                       /*                                    */
    Hd_joblnk      inreq             ; /* Jobs input queue                   */
    Hd_joblnk      lnk[4]            ; /* 4 job chain heads                  */
    USHORT         nbsys             ; /* Accumulation of "system" jobs      */
    USHORT         nbusr             ; /* Accumultaion of "user" jobs        */
                                       /*                                    */
    Hd_jfree       jfree             ; /* Free jobs head chain               */
                                       /*                                    */
    USHORT         myvl              ; /* Logical channel associated with key*/
    Dia_ctx        dia[4]            ; /* Small context for each chani head  */
                                       /*                                    */
    INT            err               ; /* Last error code value              */
    INT            retlgrcv          ; /* Last number of received bytes      */
                                       /*                                    */
  END              Key_lnk           ; /* ---------------------------------- */



/*
   2.9 Structure de lien avec la couche basse ou a droite
   ------------------------------------------------------
*/
typedef struct                         /* Pour appeler le bas niveau         */
  BEGIN                                /* ---------------------------------- */
    UINT           modd              ; /* Reference VMOS de ce contexte      */
    UINT           iod               ; /* Reference VMOS entite du dessous   */
    USHORT         numdrv            ; /* Numero driver VMIO du dessous      */
    USHORT         aut               ; /* Numero automate VMIO du dessous    */
    USHORT         vl                ; /* Numero VL VMIO du dessous          */
    const Tlcm_desc*tlcm             ; /* Ad structure Tlcm_desc du dessous  */
    Req_ctx       *rctx              ; /* Ad structure Req_ctx du dessous    */
    Key_lnk       *key               ; /* Ad structure Key_lnk du dessous    */
    USHORT         state             ; /* Etat du pipe                       */
    SHORT          first             ; /* Tete de chaine de contextes        */
  END              Next_lnk          ; /* ---------------------------------- */


/*
   2.10 Structure buffer circulaire
   --------------------------------
*/

typedef struct                         /* Buffer circulaire pour pipes       */
  BEGIN                                /* ---------------------------------- */
    UCHAR         *baseadr           ; /* Circular buffer base address       */
    UINT           segsz             ; /* Segment size in bytes              */
    UINT           segnb             ; /* Total number of segments           */
    UINT           segwrt            ; /* Number of next segment to write    */
    UINT           segrd             ; /* Number of next segment to be read  */
    UINT           segused           ; /* Number of used segments            */
  END              Circ_buf          ; /* ---------------------------------- */

/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


/* Procedures de gestion des tables de contextes ............................*/

  void  init_ctx_tab    (Tab_ctx*,Hd_ctx**,UINT,Hd_ctx*                   ,
                             UINT,UINT,UINT,UINT,UINT,UINT,UINT,INT*         );
  void  alloc_ctx       (Tab_ctx*,INT,Hd_ctx**,INT*                          );
  void  alloc_ctx_end   (Hd_ctx**,INT*                                       );
  void  post_ctx        (Hd_ctx*,INT*                                        );
  void  free_ctx        (Hd_ctx*,INT*                                        );
  void  free_ctx_end    (Hd_ctx**,INT*                                       );
  IWORD set_keylnk      (Hd_ctx*,UCHAR*                                      );
  IWORD reset_keylnk    (Hd_ctx*                                             );
  IWORD search_keylnk   (Tab_ctx*,UCHAR*,Hd_ctx**                            );
  IWORD get_ctx_adr     (Tab_ctx*,INT,Hd_ctx**                               );
  IWORD get_subctx_adr  (Hd_ctx*,Req_ctx**,Key_lnk**                         );


/* Procedures de gestion des contextes de requetes ..........................*/

  void  init_rctx       (Req_ctx*,INT,INT,INT,INT,INT,INT                 ,
                         IWORD(*)(),IWORD(*)()                               );
  IWORD chk_rctx        (Req_ctx*,Chk_req_ctx*                               );


/* Procedures d'allocation et de liberation des structures "Job" ............*/

  void  init_job        (void                                                );
  void  init_jfree      (Hd_jfree*,Job*,INT,INT                              );
  void  alloc_job       (Hd_jfree*,INT,Job**                                 );
  void  free_job        (Job*                                                );


/* Procedures de chainage ou dechainage de structures "Job" .................*/

  void  link_job        (Key_lnk*,Hd_joblnk*,Job*                            );
  void  unlink_job      (Key_lnk*,Hd_joblnk*,Job**                           );
  void  unlink_ljob     (Key_lnk*,Hd_joblnk*,Job*                            );


/* Procedures generique d'ajout de jobs dans la file d'entrees ..............*/

  void  link_setget     (Req_ctx*,Key_lnk*,INT,INT,INT,UCHAR*             ,
                         INT,INT,INT,INT,INT*                                );
  void  link_snd        (Req_ctx*,Key_lnk*,INT,INT,INT,UCHAR*             ,
                         INT,void (*)(),INT,INT,INT,INT*                     );
  void  link_rcvs       (Req_ctx*,Key_lnk*,INT,INT,INT,UCHAR*             ,
                         INT,INT,INT,INT,INT,INT*                            );
  void  link_prcv       (Req_ctx*,Key_lnk*,INT,UCHAR*,INT,INT*               );


/* Procedures de demarrage et de terminaison de jobs ........................*/

  void  inreq_job       (Req_ctx*, Key_lnk*                                  );
  void  evtend_job      (Req_ctx*,Key_lnk*,S_evt*,INT                        );
  void  push_job        (Key_lnk*                                            );
  void  pull_job        (Req_ctx*,Key_lnk*,S_evt*,INT                        );
  IWORD ready_job       (Key_lnk*,Hd_joblnk**                                );
  IWORD run_job         (Req_ctx*,Key_lnk*,Hd_joblnk*                        );


/* Procedures d'interface systeme par defaut ................................*/

  IWORD xxx_connect     (Req_ctx*,Key_lnk*,INT,UCHAR*,INT,INT,INT,INT        );
  IWORD xxx_listen      (Req_ctx*,Key_lnk*,INT,UCHAR*,INT,INT,INT,INT        );
  IWORD xxx_disconnect  (Req_ctx*,Key_lnk*,INT,INT,INT,INT,INT               );
  IWORD xxx_abort       (Req_ctx*,Key_lnk*,INT,INT,INT,INT,INT               );
  IWORD xxx_start       (Req_ctx*,Key_lnk*,INT,UCHAR*,INT,INT,INT,INT,INT    );
  IWORD xxx_stop        (Req_ctx*,Key_lnk*,INT,INT,INT,INT,INT               );
  IWORD xxx_send        (Req_ctx*,Key_lnk*,INT,UCHAR*,INT,INT,void(*)()     ,
                           INT,INT,INT                                       );
  IWORD xxx_receive     (Req_ctx*,Key_lnk*,INT,INT,INT,UCHAR*,INT,INT,INT,INT);
  IWORD xxx_setval      (Req_ctx*,Key_lnk*,INT,UCHAR*,INT,INT,INT,INT        );
  IWORD xxx_getval      (Req_ctx*,Key_lnk*,INT,UCHAR*,INT,INT,INT,INT        );
  IWORD xxx_rpc         (Req_ctx*,Key_lnk*,INT,const UCHAR*                 ,
                         INT,INT,INT,INT,INT,INT                             );
  IWORD xxx_io          (Req_ctx*,Key_lnk*,Job*,INT                          );
  IWORD vvv_write       (Req_ctx*,Key_lnk*,INT,UCHAR*,INT                    );
  IWORD vvv_read        (Req_ctx*,Key_lnk*,INT,UCHAR*,INT,INT*               );


/* Procedures de traitements d'automates VMIO generiques ....................*/

  IWORD x_init          (void                                                );
  IWORD x_end           (void                                                );
  IWORD x_alloc_dev     (Req_ctx*,INT,INT,INT,const USHORT*,INT*             );
  IWORD x_free_dev      (Req_ctx*,INT,INT,INT*                               );
  IWORD x_open_dev      (Req_ctx*,void (*)(),UINT                            );
  IWORD x_close_dev     (Req_ctx*,UINT                                       );
  IWORD x_alloc_sub     (Req_subctx*,INT,INT,INT,INT,
                         const void*,void(*)(),INT,INT,INT*                  );
  IWORD x_free_sub      (Req_subctx*,INT,INT,INT*                            );
  IWORD x_connect       (Req_ctx*,Key_lnk*,INT                               );
  IWORD x_listen        (Req_ctx*,Key_lnk*,INT                               );
  IWORD x_write         (Req_ctx*,Key_lnk*,INT                               );
  IWORD x_read          (Req_ctx*,Key_lnk*,INT                               );
  IWORD x_setval        (Req_ctx*,Key_lnk*,INT                               );
  IWORD x_getval        (Req_ctx*,Key_lnk*,INT                               );
  IWORD x_setval_sub    (Req_ctx*,Key_lnk*,INT,Req_subctx*                   );
  IWORD x_getval_sub    (Req_ctx*,Key_lnk*,INT,Req_subctx*                   );
  IWORD x_rpc           (Req_ctx*,Key_lnk*,INT                               );
  IWORD x_evtend        (Req_ctx*,Key_lnk*,INT                               );
  IWORD x_pstart        (Req_ctx*,Key_lnk*,INT,Circ_buf*,Next_lnk*           );
  IWORD x_rwrite        (Req_ctx*,Key_lnk*,INT,Circ_buf*                     );
  IWORD x_req_clear     (Req_ctx*,Key_lnk*,INT                               );


/* Procedures sous-programmes de depot d'evenements .........................*/
 
  void  evt_resperror   (Req_ctx*,INT,UCHAR*,INT,INT                         );
  void  evt_respjob     (Req_ctx*,Job*,INT,INT,UCHAR*,INT                    );
  void  evt_ioend       (Req_ctx*,Key_lnk*,Job*,INT                          );
  void  evt_resprctx    (Req_ctx*,INT,INT,UCHAR*,INT,UINT,UINT               );


/* Procedures d'interface systeme avec la couche du bas ou a droite .........*/

  void  init_next_lnk   (Next_lnk*,UINT,UINT,INT,INT                         );
  IWORD connect_next    (Next_lnk*,UCHAR*,INT,INT,INT,INT                    );
  IWORD listen_next     (Next_lnk*,UCHAR*,INT,INT,INT,INT                    );
  IWORD disconnect_next (Next_lnk*,INT,INT,INT,INT                           );
  IWORD abort_next      (Next_lnk*,INT,INT,INT                               );
  IWORD start_next      (Next_lnk*,UCHAR*,INT,INT,INT,INT,INT                );
  IWORD stop_next       (Next_lnk*,INT,INT,INT,INT                           );
  IWORD send_next       (Next_lnk*,UCHAR*,INT,INT,void(*)(),INT,INT,INT      );
  IWORD receive_next    (Next_lnk*,INT,INT,INT,INT                           );
  IWORD setval_next     (Next_lnk*,UCHAR*,INT,INT,INT,INT                    );
  IWORD getval_next     (Next_lnk*,UCHAR*,INT,INT,INT,INT                    );
  IWORD rpc_next        (Next_lnk*,UCHAR*,INT,INT,INT,INT,INT,INT            );
  IWORD io_next         (Next_lnk*,Job*,INT                                  );
  IWORD aut2drv         (INT                                                 );
  IWORD drv2aut         (INT                                                 );
  void  search_name2    (CHAR*,CHAR*,INT*                                    );


#endif                                 /* End of REQ_LIB_H.                  */
