/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                         File "stm32/include/stm32.h"       */
/*  Constantes and definitions for specific stm32 board.                     */
/*                                                                           */
/*  Author : JYA                        Last modification : 10/07/2020       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef STM32_H                        /* Si le fichier n'est pas deja       */
#define STM32_H                        /* inclus, inclure son contenu        */


/*
    1    Defines
    ____________
*/


/*
    2    Structures and typedefs
    ____________________________
*/

typedef struct                         /* "FLASH Interface" register page 98"*/
   BEGIN                               /* ---------------------------------- */
     UINT           acr              ; /* 0x0000 Flash Access Control        */
     UINT           key              ; /* 0x0004 Key reg to access to acr reg*/
     UINT           optkey           ; /* 0x0008 Option Key reg              */
     UINT           status           ; /* 0x000C Status of flash operations  */
     UINT           ctrl             ; /* 0x0010 Cfg. & Start Flash operation*/
     UINT           optctrl          ; /* 0x0014 Cfg user option bytes       */
   END              Flash            ; /* ---------------------------------- */


typedef struct                         /* SYSCFG registers                   */
   BEGIN                               /* ---------------------------------- */
     UINT           memrmp           ; /* 0x00 Memmory remap register        */
     UINT           pmc              ; /* 0x04 Peripheral mode config.       */
     UINT           extic[4]         ; /* 0x08 External interrupt config. 1  */
                                     ; /* 0x0C External interrupt config. 2  */
                                     ; /* 0x10 External interrupt config. 3  */
                                     ; /* 0x14 External interrupt config. 4  */
     UINT           res[2]           ; /*                                    */
#ifdef VM_STM32M4
     UINT           cmpc             ; /* 0x20 Compensation cell control     */
#endif
   END              Syscfg           ; /* ---------------------------------- */

typedef struct                         /* Systick, see ARMv7-M Architecture  */
                                       /* Reference Manual Par B, page       */
                                       /* B3-745                             */
  BEGIN                                /* ---------------------------------- */
    UINT            cs               ; /* Control and Status                 */
    UINT            rv               ; /* Reload Value                       */
    UINT            cv               ; /* Current Value                      */
    UINT            calib            ; /* Calibration                        */
  END               Systick          ; /* ---------------------------------- */

typedef struct                         /* "EXTI" registers                   */
   BEGIN                               /* ---------------------------------- */
     UINT           im               ; /* 0x00 Interrupt mask                */
     UINT           em               ; /* 0x04 Event mask                    */
     UINT           rts              ; /* 0x08 Rising trigger selection      */
     UINT           fts              ; /* 0x0C Falling trigger selection     */
     UINT           swie             ; /* 0x10 Software interrupt event      */
     UINT           pr               ; /* 0x14 Pending (status) register     */
   END              Exti             ; /* ---------------------------------- */

typedef struct                         /* Registres timer                    */
  BEGIN                                /* ---------------------------------- */
    UINT            cr1              ; /* 0x00 Control Register 1            */
    UINT            cr2              ; /* 0x04 Control Register 2            */
    UINT            smc              ; /* 0x08 Slave-mode control            */
    UINT            die              ; /* 0x0C DMA/Interrupt enable          */
    UINT            sr               ; /* 0x10 Status register               */
    UINT            eg               ; /* 0x14 Event generation              */
    UINT            ccm1             ; /* 0x18 Capture/Compare mode 1        */
    UINT            ccm2             ; /* 0x1C Capture/Compare mode 2        */
    UINT            cce              ; /* 0x20 Capture/Compare enable        */
    UINT            cnt              ; /* 0x24 Counter                       */
    UINT            psc              ; /* 0x28 Prescaler                     */
    UINT            ar               ; /* 0x2C Auto-reload                   */
    UINT            rc               ; /* 0x30 Repetition counter            */
    UINT            cc1              ; /* 0x34 Capture/Compare 1             */
    UINT            cc2              ; /* 0x38 Capture/Compare 2             */
    UINT            cc3              ; /* 0x3C Capture/Compare 3             */
    UINT            cc4              ; /* 0x40 Capture/Compare 4             */
    UINT            bdt              ; /* 0x44 Break and dead-time           */
    UINT            dc               ; /* 0x48 DMA control                   */
    UINT            dma              ; /* 0x4C DMA address                   */
  END               Tim              ; /* ---------------------------------- */

typedef struct                         /* RCC                                */
  BEGIN                                /* ---------------------------------- */
    UINT            cr               ; /* 00 Clock control Register          */
#ifdef VM_STM32M3
    UINT            icscr            ; /* 04 Internal Clock sources calibrati*/
    UINT            cfgr             ; /* 08 Clock configuration             */
    UINT            cir              ; /* 0C Clock interrupt                 */
    UINT            ahbrst           ; /* 10 \                               */
    UINT            apb2rst          ; /* 14 | Reset                         */
    UINT            apb1rst          ; /* 18 /                               */
    UINT            ahben            ; /* 1C \                               */
    UINT            apb2en           ; /* 20 | Clock enable                  */
    UINT            apb1en           ; /* 24 /                               */
    UINT            ahblpen          ; /* 28 \                               */
    UINT            apb2lpen         ; /* 2C | Clock enable in low-power mode*/
    UINT            apb1lpen         ; /* 30 /                               */
    UINT            csr              ; /* 34 Control & Status                */
#endif
#ifdef VM_STM32M4
    UINT            pllcfg           ; /* 04 PLL configuration               */
    UINT            cfgr             ; /* 08 Clock configuration             */
    UINT            cir              ; /* 0C Clock interrupt                 */
    UINT            ahb1rst          ; /* 10 \                               */
    UINT            ahb2rst          ; /* 14 |                               */
    UINT            ahb3rst          ; /* 18 | Reset                         */
    UINT            res1[1]          ; /*                                    */
    UINT            apb1rst          ; /* 20 |                               */
    UINT            apb2rst          ; /* 24 /                               */
    UINT            res2[2]          ; /*                                    */
    UINT            ahb1en           ; /* 30 \                               */
    UINT            ahb2en           ; /* 34 |                               */
    UINT            ahb3en           ; /* 38 | Clock enable                  */
    UINT            res3[1]          ; /*                                    */
    UINT            apb1en           ; /* 40 |                               */
    UINT            apb2en           ; /* 44 /                               */
    UINT            res4[2]          ; /*                                    */
    UINT            ahb1lpen         ; /* 50 \                               */
    UINT            ahb2lpen         ; /* 54 |                               */
    UINT            ahb3lpen         ; /* 58 | Clock enable in low-power mode*/
    UINT            res5[1]          ; /*                                    */
    UINT            apb1lpen         ; /* 60 |                               */
    UINT            apb2lpen         ; /* 64 /                               */
    UINT            res6[2]          ; /*                                    */
    UINT            bdc              ; /* 70 Backup domain control           */
    UINT            csr              ; /* 74 Control & Status                */
    UINT            res7[2]          ; /*                                    */
    UINT            sscg             ; /* 80 Spread spectrum clock generation*/
    UINT            plli2scfg        ; /* 84 PLLI2S configuration            */
    UINT            pllsaicfg        ; /* 88 PLL configuration               */
    UINT            dckcfg           ; /* 8C Dedicated Clock Configuration   */
#endif
  END               Rcc_ctrl         ; /* ---------------------------------- */

typedef struct                         /* Controleur RTC                     */
  BEGIN                                /* ---------------------------------- */
    UINT            tr               ; /* Shadow time                        */
    UINT            dr               ; /* Shadow date                        */
    UINT            cr               ; /* Control                            */
    UINT            isr              ; /* Initialisation and status          */
    UINT            prer             ; /* Prescaler                          */
    UINT            wtur             ; /* Wakeup timer unit                  */
    UINT            calibr           ; /* Calibration                        */
    UINT            alrmar           ; /* Alarm A                            */
    UINT            alrmbr           ; /* Alarm B                            */
    UINT            wpr              ; /* Write protection                   */
    UINT            ssr              ; /* Sub-second                         */
    UINT            shiftr           ; /* Shift                              */
    UINT            tstr             ; /* Time-stamp time                    */
    UINT            tsdr             ; /* Time-stamp date                    */
    UINT            tsssr            ; /* Time-stamp sub-second              */
    UINT            calr             ; /* Calibration register               */
    UINT            tafcr            ; /* Tamper & alter function config     */
    UINT            alrmassr         ; /* Alarm A sub_second                 */
    UINT            alrmbssr         ; /* Alarm B sub_second                 */
    UINT            bkpr             ; /* Backup                             */
  END               Rtc              ; /* ---------------------------------- */

typedef struct                         /* Controleur GPIO                    */
  BEGIN                                /* ---------------------------------- */
    UINT            mode             ; /* Mode : in, out, alt, anl           */
    UINT            otyp             ; /* Type d'out: push-pull, open-drain  */
    UINT            ospd             ; /* Out speed : very l, low, med, high */
    UINT            pupd             ; /* Pull-up, pull-down                 */
    UINT            id               ; /* Input data                         */
    UINT            od               ; /* Output data                        */
    UINT            bsr              ; /* Bit set reset                      */
    UINT            lck              ; /* GPIO lock                          */
    UINT            af[2]            ; /* Alternate function low / high      */
    UINT            br               ; /* Bit reset                          */
  END               Gpio             ; /* ---------------------------------- */

typedef struct                         /* DMA Channel registers              */
  BEGIN                                /* ---------------------------------- */
    UINT            csc              ; /* - 1xx: 0x08 Channel configuration  */
                                       /* - 4xx: 0x10 Stream configuration   */
    UINT            csndt            ; /* - 1xx: 0x0C Counter                */
                                       /* - 4xx: 0x14 Counter                */
    void*           cspa             ; /* - 1xx: 0x10 Peripheral address     */
                                       /* - 4xx: 0x18 Peripheral address     */
#ifdef VM_STM32M3
    void*           cma              ; /* 0x14 Memory address                */
    UINT            res              ; /* 0x18                               */
#endif
#ifdef VM_STM32M4
    void*           sma0             ; /* 0x1C Memory address 0              */
    void*           sma1             ; /* 0x20 Memory address 1              */
    UINT            sfc              ; /* 0x24 FIFO control                  */
#endif
  END               DmaSdev          ; /* ---------------------------------- */

typedef struct                         /* DMA                                */
  BEGIN                                /* ---------------------------------- */
#ifdef VM_STM32M3
    UINT            is               ; /* 0x00 Interrupt Status              */
    UINT            ifc              ; /* 0x04 Interrupt Flag clear          */
    DmaSdev         sdev[7]          ; /* Interrupt Status                   */
#endif
#ifdef VM_STM32M4
    UINT            is[2]            ; /* 0x00 Low Interrupt Status          */
                                     ; /* 0x04 High Interrupt Status         */
    UINT            ifc[2]           ; /* 0x08 Low Interrupt Flag clear      */
                                       /* 0x0C High Interrupt Flag clear     */
    DmaSdev         sdev[8]          ; /* Interrupt Status                   */
#endif
  END               Dma              ; /* ---------------------------------- */

typedef struct                         /* Register USART                     */
  BEGIN                                /* ---------------------------------- */
    UINT            cr1              ; /* 0x00 I2C control register 1        */
    UINT            cr2              ; /* 0x04 I2C control register 2        */
    UINT            oar1             ; /* 0x08 Own address register 1        */
    UINT            oar2             ; /* 0x0C Own address register 2        */
    UINT            dr               ; /* 0x10 Data register                 */
    UINT            sr1              ; /* 0x14 Status register 1             */
    UINT            sr2              ; /* 0x18 Status register 2             */
    UINT            ccr              ; /* 0x1C Clock control register        */
    UINT            trise            ; /* 0x20 TRISE register                */
#ifdef VM_STM32M4
    UINT            fltr             ; /* 0x24 FLTR register                 */
#endif
  END               I2c              ; /* ---------------------------------- */

typedef struct                         /* Register USART                     */
  BEGIN                                /* ---------------------------------- */
    UINT            sr               ; /* 0x00 Status Register               */
    UINT            dr               ; /* 0x04 Data Register                 */
    UINT            brr              ; /* 0x08 Baudrate Generator Register   */
    UINT            cr1              ; /* 0x0C Control Register 1            */
    UINT            cr2              ; /* 0x10 Control Register 2            */
    UINT            cr3              ; /* 0x14 Control Register 3            */
    UINT            gtpr             ; /* 0x18 Guard time & prescaler        */
  END               Asc              ; /* ---------------------------------- */

typedef struct                         /* Register SPI/I2S                   */
  BEGIN                                /* ---------------------------------- */
    UINT            cr1              ; /* 0x00 Control register 1            */
    UINT            cr2              ; /* 0x04 Control register 2            */
    UINT            sr               ; /* 0x08 Status register               */
    UINT            dr               ; /* 0x0C Data register                 */
    UINT            crcpr            ; /* 0x10 CRC polynomial register       */
    UINT            rxcrc            ; /* 0x14 Reception CRC                 */
    UINT            txcrc            ; /* 0x18 Transmission CRC              */
    UINT            i2scfgr          ; /* 0x1C I2S configuration register    */
    UINT            i2sspr           ; /* 0x20 I2S prescaler register        */
  END               Spi              ; /* ---------------------------------- */

typedef struct                         /* Register DAC                       */
  BEGIN                                /* ---------------------------------- */
    UINT            cr               ; /* 0x00 Control register              */
    UINT            swtrigr          ; /* 0x04 Software trigger register     */
    UINT            dhr12r1          ; /* 0x08 Channel 1 12bits right-aligned*/
                                       /*      data holding register         */
    UINT            dhr12l1          ; /* 0x0C Channel 1 12bits left-aligned */
                                       /*      data holding register         */
    UINT            dhr8r1           ; /* 0x10 Channel 1 8bits right-aligned */
                                       /*      data holding register         */
    UINT            dhr12r2          ; /* 0x14 Channel 2 12bits right-aligned*/
                                       /*      data holding register         */
    UINT            dhr12l2          ; /* 0x18 Channel 2 12bits left-aligned */
                                       /*      data holding register         */
    UINT            dhr8r2           ; /* 0x1C Channel 2 8bits right-aligned */
                                       /*      data holding register         */
    UINT            dhr12rd          ; /* 0x20 Dual ch 12bits right-aligned  */
                                       /*        data holding                */
    UINT            dhr12ld          ; /* 0x24 Dual ch 12bits left-aligned   */
                                       /*      data holding                  */
    UINT            dhr8rd           ; /* 0x28 Dual ch 8bits right-aligned   */
                                       /*      data holding                  */
    UINT            dor1             ; /* 0x2C Channel 1 data output reg     */
    UINT            dor2             ; /* 0x30 Channel 2 data output reg     */
    UINT            sr               ; /* 0x34 Status register               */
  END               Dac              ; /* ---------------------------------- */

typedef struct                         /* Registers specific to each ADC     */
  BEGIN                                /* ---------------------------------- */
    UINT            sr               ; /* 0x00 status register               */
    UINT            cr1              ; /* 0x04 control register 1            */
    UINT            cr2              ; /* 0x08 control register 2            */
    UINT            smpr1            ; /* 0x0C sample time register 1        */
    UINT            smpr2            ; /* 0x10 sample time register 2        */
#ifdef VM_STM32M3
    UINT            smpr3            ; /* 0x14 sample time register 3        */
    UINT            jofr[4]          ; /* 0x18 -> 0x24 injected channel data */
                                       /*              offset register       */
    UINT            htr              ; /* 0x28 watchdog higher threshold reg */
    UINT            ltr              ; /* 0x2C watchdog lower threshold reg  */
    UINT            sqr[5]           ; /* 0x30 -> 0x40 regular sequence reg  */
    UINT            jsqr             ; /* 0x44 injected sequence register    */
    UINT            jdr[4]           ; /* 0x48 -> 0x54 injected data register*/
    UINT            dr               ; /* 0x58 regular data register         */
    UINT            smpr0            ; /* 0x5C sample time register 0        */
    UINT            pad1[160]        ; /* 0x60 -> 0xFF padding               */
#endif
#ifdef VM_STM32M4
    UINT            jofr[4]          ; /* 0x14 -> 0x20 injected channel data */
                                       /*              offset register       */
    UINT            htr              ; /* 0x24 watchdog higher threshold reg */
    UINT            ltr              ; /* 0x28 watchdog lower threshold reg  */
    UINT            sqr[3]           ; /* 0x2C -> 0x34 regular sequence reg  */
    UINT            jsqr             ; /* 0x38 injected sequence register    */
    UINT            jdr[4]           ; /* 0x3C -> 0x48 injected data register*/
    UINT            dr               ; /* 0x4C regular data register         */
    UINT            pad1[176]        ; /* 0x50 -> 0xFF padding               */
#endif
  END               AdcNum           ; /* ---------------------------------- */

typedef struct                         /* Registers ADC                      */
  BEGIN                                /* ---------------------------------- */
#ifdef VM_STM32M3
    AdcNum          adc[1]           ; /* One ADC                            */
    UINT            pad1[512]        ; /* 0x0100 -> 0x02FC padding           */
    UINT            csr              ; /* 0x300 common status register       */
    UINT            ccr              ; /* 0x304 common control register      */
#endif
#ifdef VM_STM32M4
    AdcNum          adc[3]           ; /* 3 ADCs                             */
    UINT            csr              ; /* 0x300 common status register       */
    UINT            ccr              ; /* 0x304 common control register      */
    UINT            cdr              ; /* 0x308 common regular data register */
                                       /*       for dual and triple modes    */
#endif
  END               Adc              ; /* ---------------------------------- */



/*
    3    Public variables
    _____________________
*/

#ifdef TOTO

extern const Kbd_desc rcu_desc_stm32        ; /* RCU  low level interface    */

#endif

/*
    4    Prototypes of public procedures
    ____________________________________
*/


extern IWORD set_pio             (INT,INT,INT                                );
extern IWORD get_pio             (INT,INT,INT*                               );

extern IWORD handlerRCU_stm32    (IWORD,IWORD                                );


#endif                                 /* #ifndef STM32_H                    */
