/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/vmio.h"                */
/*  Prototypes of general I/O container procedures.                          */
/*                                                                           */
/*  Author : JYA                        Last modification : 09/20/2020       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef VMIO_H                         /* If this file is not already        */
#define VMIO_H                         /* included, include it.              */


/*
    1    Defines
    ____________
*/

#define ADD_EVT_S 0x01
#define DEL_EVT_S 0x10
#define BEG_TRT_S 0x20
#define ACT_TRT_S 0x21


/*
    2    Structures and typedefs
    ____________________________
*/


/*
    3    Public variables
    _____________________
*/

extern S_evt           c_evt         ; /* Evenement courant                  */
extern USHORT          numauto_s     ; /* Numero de l'automate concerne      */
extern USHORT          voielog_s     ; /* Numero VL courante                 */
extern USHORT          condition_s   ; /* <> 0 si condition positionnee      */
extern IWORD           etat_s        ; /* Etat courant de l'automate         */
extern SHORT           code_s        ; /* Code evenement entrant             */

extern const INT       fan_idx       ; /* FAN   index                        */
extern const INT       fau_idx       ; /* FAU   index                        */
extern const INT       fal_idx       ; /* FAL   index                        */

/*
    4    Prototypes of public procedures
    ____________________________________
*/


  IWORD init_auto_s         (IWORD                                           );
  IWORD end_auto_s          (IWORD                                           );
  void  iniaut_s            (IWORD,const S_trans *, const USHORT *,USHORT *  );
  IWORD vmio_s              (IWORD,IWORD                                     );
  IWORD trigto_s            (IWORD,IWORD                                     );

  IWORD putevt_vmio         (S_evt*                                          );
  IWORD putevt_vmio_urg     (S_evt*                                          );
  IWORD send_vmio           (S_evt*                                          );
  IWORD clrevt_vmio         (INT,UINT,UINT,UINT, IWORD (*)()                 );
  void  relaut_s            (UINT                                            );

  UINT  setto_s             (UINT,INT,IWORD,IWORD                            );
  IWORD clearto_s           (INT,UINT                                        );
  IWORD clear_autvlto_s     (INT,INT                                         );

  IWORD stoevt_s            (S_evt*                                          );
  IWORD getevt_s            (IWORD,IWORD*,IWORD,IWORD,S_evt*                 );
  IWORD clrevt_s            (IWORD,IWORD (*)()                               );

  IWORD add_route_s         (INT,INT,INT,INT,INT,INT,UINT,UINT,INT,UINT*     );
  IWORD del_route_s         (UINT*,INT, IWORD (*)()                          );
  IWORD set_route_s         (UINT*,INT,INT,INT                               );
  IWORD route_evt_s         (S_evt*                                          );
  IWORD del_evt_route_s     (INT,IWORD (*)()                                 );

  IWORD trien_s             (INT                                             );
  IWORD get_autvl           (INT*,INT*                                       );
  IWORD send_resp           (INT,INT,UCHAR*,INT,UINT,UINT                    );
  void  set_respinfo        (Resp_info*                                      );
  void  reset_respinfo      (Resp_info*                                      );
  IWORD send_respinfo       (INT,INT,UCHAR*,INT,UINT,UINT,Resp_info*         );

#endif                                 /* End of VMIO_H.                     */
