/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/vmk.h"                 */
/*  Constants and definitions for generic engine automaton.                  */
/*                                                                           */
/*  Author : JYA                        Last modification : 08/16/2018       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Constantes de definition des files d'attente du VMOS
            1.2  Constantes de dimensionnements de tables du VMK
            1.3  Constantes autres du VMK

         2  Structures and typedefs
            2.1  Structures pour les files d'attente internes du VMK
            2.2  S_evt event structure, used by VMK and VMIO
            2.3  S_trans transition structure, used by VMK and VMIO
            2.4  S_aut Automaton structure, used by VMK and VMIO
            2.5  Structure Time-out du VMK
            2.6  Structure Time-out du VMIO
            2.7  Structure Route du VMK et du VMIO
            2.8  Structure condition de depot du VMK
            2.9  Structure zone designable sur l'ecran
            2.10 Structure de description d'un groupe de taches
            2.11 Structure contexte de haut niveau d'une tache
            2.12 Structure contexte d'un semaphore de tache

         3  Public variable

         4  Prototypes of public procedures
            4.1  Prototypes proceduresde la boucle principale du VMK
            4.2  Prototypes proceduresautomate de base
            4.3  Prototypes procedurespour les conditions sur sessid
            4.4  Prototypes procedurespour les routes
            4.5  Prototypes procedurespour les temporisateurs
            4.6  Prototypes proceduresvides
            4.7  Prototypes proceduresde gestion des taches et des piles
            4.8  Prototypes procedurespour semaphores
            4.9  Prototypes procedurespour mise en veille
            4.10 Prototypes procedurespour debugger un logiciel embarque
            4.11 Prototypes proceduresdestinees au debugger de scripts
            4.12 Prototypes procedurescouche basse du VMK


*/


#ifndef MOTEUR_H                       /* If this file is not already        */
#define MOTEUR_H                       /* included, include it.              */


/*
    1    Defines
    _____________
*/

/*
   1.1 Constantes de definition des numeros des files d'attente du VMOS
   --------------------------------------------------------------------

   La file 0 est la file des entrees vides.
   De meme la priorite 0 est inutilisee.
*/

#define NFA_FREEEVT      0             /* File des entrees vides.            */


/* La file 1 est utilisee pour le debugger. Elle n'est pas utilisee 
   actuellement. Elle est necessaire pour que les evenements stockes par le 
   debugger soient systematiquement executes avant tous les autres. Cette file
   a une priorite fixe, egale a 1.
*/

#define NFA_DBG_P0       1             /* File usage debugger.               */
#define PRIO_DBG_P0      0             /* Priorite de cette file.            */


/* Les files 2 a 15 sont reservees a l'automate de gestion des taches.
   Ces files ont une priorite variable, entre 2 et 15.
*/

#define NFA_TSK_FIRST    2             /* 1ere file des taches               */
#define NFA_TSK_LAST    15             /* Derniere file des taches           */
#define NFA_TSK_NB      14             /* Nb de files pour AUT_TASK.         */
#define PRIO_TSK_NB     14             /* Nb priorite pour les files de tache*/
#define PRIO_TSK_MAX     2
#define PRIO_TSK_MIN    15

/* La file standard STD_P4 est utilisee dans les cas tres particuliers :
   . Lorsqu'on est en train de charger un ensemble qui reference un ensemble 
     qu'on est en train de decharger, il faut se placer en attente de la fin
     du dechargement de l'ensemble reference pour pouvoir relancer son 
     chargement et ainsi terminer le chargement de celui qui le reference.
   . Le module ca_api.c declare attendre les evenements carte a puce dans cette
     file d'attente car il doit etre prevenu avant tous les modules specifiques
     au controle d'acces de l'insertion et de l'extraction de la carte a puce.
   . On passe aussi par cette file pour mettre en place les cles de dechif-
     frement, de maniere a ce qu'elles prenent effet immediatement, avant tout
     autre traitement VMOS.
    Cette file a une priorite fixe et egale a 16.
*/

#define NFA_STD_P4      16             /* File usage standard : priorite 4   */
#define PRIO_STD_P4      1

/* La file standard serveur est utilisee par les modules applicatifs qui le
   declarent specifiquement. Cette file va etre utilisee dans le cas ou une
   application serveur et une application client sont executees par le meme
   VMOS. Elle permet aux threads du serveur qui le necessitent de se
   declarer explicitement plus prioritaires que les autres threads.
   Cette file a une priorite fixe et egale a 17.
*/

#define NFA_STD_PS      17             /* File usage standard : serveur      */
#define PRIO_STD_PS     16             /* Priorite de cette file.            */


/* La file panel PAN_P2 est utilisee pour MHEG5.
   Cette file a une priorite fixe et egale a 18.
*/

#define NFA_PAN_P2      18             /* File usage panel    : priorite 2   */
#define PRIO_PAN_P2     17             /* Priorite de cette file.            */


/* La file panel PAN_P1 est utilisee pour stocker les requetes internes
   envoyees aux panels. A chaque fois qu'une requete est envoyee au
   gestionnaire de panels durant le traitement d'une requete, elle est
   consideree comme une requete interne, c'est a dire que son traitement
   doit etre execute avant de considerer que le gestionnaire de panels a fini 
   de traiter la requete externe en cours.
   Cette file a une priorite fixe et egale : 19.
*/

#define NFA_PAN_P1      19             /* File usage panel    : priorite 1   */
#define PRIO_PAN_P1     18             /* Priorite de cette file.            */


/* La file 20 est utilises par les differents contextes d'execution des
   scripts. Elle a la priorite 20.
*/

#define NFA_SCR         20
#define PRIO_SCR        19             /* Priorite de cette file.            */


/* La file STD_P3 est utilisee pour tous les evenements applicatifs et
   internes sauf pour les applications qui declarent explicitement qu'ils
   veulent utiliser une file plus prioritaire.
   Cette file a la plus grande taille.
   Cette file a une priorite fixe et egale a 21.
*/

#define NFA_STD_P3      21
#define PRIO_STD_P3     20             /* Priorite de cette file.            */


/* La file panel PAN_P0 est utilisee pour stocker les requetes externes
   envoyees aux panels. Par exemple, lorsqu'un script recoit un evenemenent et 
   decide de modifier le texte associe a un widget d'un panel, le script 
   appelle la procedure set_widget_text() qui depose une requete dans la file 
   d'attente PAN_P0. Le gestionnaire de panels controle l'utilisation de cette 
   file d'attente. A chaque fois qu'il est pret, le gestionnaire autorise la 
   lecture d'un evenement de cette file d'attente. L'evenement extrait de 
   PAN_P0 est place dans PAN_P1 puis le gestionnaire de panels interdit de 
   nouveau la lecture de la file PAN_P0 et ce jusqu'a ce que la file d'attente 
   PAN_P1 soit de nouveau vide, c'est a dire jusqu'a ce que le gestionnaire de 
   panels ait termine de traiter la requete externe.
   Cette file a une priorite fixe et egale a 22.
*/

#define NFA_PAN_P0      22             /* File usage panel                   */
#define PRIO_PAN_P0     21             /* Priorite de cette file.            */



/*
   1.2 Constantes de dimensionnements de tables du VMK
   ---------------------------------------------------
*/

#define NBFAI           23             /* Nombre de files d'attentes         */
#define NBPRIO          22             /* Nombre total de priorites.         */


#define NLIMEVT         4              /* nb max struct. de limitation.      */
#define MAX_INDEX       10             /* nb max proc. de routage. (event.c) */


/*
   1.3  Constantes autres du VMK
   -----------------------------
*/

#define STCYCLE    200                 /* Temps de cycle en millisecodnes    */
#define SMINQUANT   10                 /* Quantum minimum en millisecondes   */

/*
    2    Structures and typedefs
    ____________________________
*/



/*
   2.1 Structures pour les files d'attente internes du VMK
   -------------------------------------------------------
*/

typedef struct                         /* Une file d'attente.                */
  BEGIN                                /* ---------------------------------- */
    USHORT      nbevt                ; /* Nombre d'evenements dans la file   */
    USHORT      first                ; /* Indice du premier evt dans la file */
    USHORT      last                 ; /* Indice du dernier evt dans la file */

    UCHAR       nxtfa                ; /* File qui suit dans le chainage et  */
    UCHAR       prvfa                ; /* file qui precede (sur la priorite).*/
    UCHAR       prio                 ; /* Priorite de la file.               */

    UCHAR       hfilt                ; /* Indice 1ere structure de "fai_filt"*/
    USHORT      nbuevt               ; /* Nombre d'evenements non verrouilles*/
  END           Fai_vmk              ; /* ---------------------------------- */


typedef struct                         /* Une priorite de file.              */
  BEGIN                                /* ---------------------------------- */
    USHORT      nbevt                ; /* Nb d'evt dans les files de la prio */
    UCHAR       nxtfa                ; /* Prochaine file a examiner.         */
    UCHAR       nbfa                 ; /* Nb de files pour cette priorite.   */
  END           Fai_prio             ; /* ---------------------------------- */


typedef struct                         /* Contexte de liste d'evts attendus  */
  BEGIN                                /* ---------------------------------- */
    UCHAR       alloc                ; /* Indique si cette entree est allouee*/
    UCHAR       mode                 ; /* Liberation  0:manuelle  1:auto     */
    SHORT       aut                  ; /* Automate ayant cree la liste       */
    USHORT      vl                   ; /* VL       ayant cree la liste       */
    INT         nbevt                ; /* Nombre d'evenements dans la liste  */
    UCHAR      *evt                  ; /* Ad. du buffer contenant ces evts   */
  END           Waitevtlist          ; /* ---------------------------------- */

typedef struct                         /* Contexte de filtre d'evenement     */
  BEGIN                                /* ---------------------------------- */
    UCHAR       alloc                ; /* Indique si cette entree est allouee*/
    UCHAR       sat                  ; /* Indique si le filtre est satisfait */
    SHORT       aut                  ; /* Automate ayant verrouille la file. */
    USHORT      vl                   ; /* Vl ayant verrouille la file.       */
    USHORT      next                 ; /* Chainage sur structure suivante    */
    Waitevtlist *evtlist             ; /* Contexte de la liste d'evenements  */
  END           Fai_filt             ; /* ---------------------------------- */

/*
   2.2  S_evt event structure, used by VMK and VMIO
   ------------------------------------------------

   This structure is the most important one of the OS, meaning that 
   modifications can only be made by the OS architect. It is used by both
   the VMK FSM engine (application container) and the VMIO FSM engine (IO
   container). All the App container queues are arrays or linked lists of
   S_evt structures. The three input queuesof the VMIO are linked lists of
   this same structure.

*/

typedef struct s_evt                   /* One event                          */
  BEGIN                                /* ---------------------------------- */
    USHORT      aut                  ; /* Target automaton number            */
    USHORT      vl                   ; /* Target logical way number          */
    USHORT      node                 ; /* Target node number (0 local)       */
                                       /*                                    */
    USHORT      code                 ; /* Event code                         */
    INT         reserve              ; /* IOD value                          */
    UCHAR       *adresse             ; /* Data buffer address                */
    INT         longueur             ; /* Data buffer length                 */
    UINT        flags                ; /* Interface procedure flags parameter*/
    UINT        res2                 ; /* Additionnal reserve field          */
                                       /*                                    */
    USHORT      reqemet              ; /* Request originator automaton       */
    USHORT      reqevl               ; /* Request originator logical way     */
    USHORT      reqnode              ; /* Request originator node            */
    USHORT      reqid                ; /* User request identifier            */
    USHORT      sessid               ; /* Session identifier                 */
    UINT        rteid                ; /* Route identifier                   */
                                       /*                                    */
    USHORT      emet                 ; /* Event sender  automaton            */
    USHORT      evl                  ; /* Event sender  logical way          */
    UINT        cptevt               ; /* Event unique identifier/counter    */
    UINT        dbg                  ; /* Redundancy data for debug          */
    USHORT      next                 ; /* Link to the next queued event      */
  END           S_evt                ; /* ---------------------------------- */

typedef struct                         /* A conserver pour un evt RESP_      */
  BEGIN                                /* ---------------------------------- */
    USHORT      reqemet              ; /* reqemet  de la reponse             */
    USHORT      reqevl               ; /* reqevl   de la reponse             */
    USHORT      reqnode              ; /* reqnode  de la reponse             */
    USHORT      reqid                ; /* reqid    de la reponse             */
    USHORT      sessid               ; /* sessid   de la reponse             */
  END           Resp_info            ; /* ---------------------------------- */


/* Offset des champs de la structure S_evt pour les codes EMU32 ------------ */

#define OF7_AUT   of7(S_evt      ,aut           ) /*                         */
#define OF7_VL    of7(S_evt      ,vl            ) /*                         */
#define OF7_CODE  of7(S_evt      ,code          ) /*                         */
#define OF7_RES   of7(S_evt      ,reserve       ) /*                         */
#define OF7_ADR   of7(S_evt      ,adresse       ) /*                         */
#define OF7_LG    of7(S_evt      ,longueur      ) /*                         */
#define OF7_FLAG  of7(S_evt      ,flags         ) /*                         */
#define OF7_RQEM  of7(S_evt      ,reqemet       ) /*                         */
#define OF7_RQVL  of7(S_evt      ,reqevl        ) /*                         */
#define OF7_RQID  of7(S_evt      ,reqid         ) /*                         */
#define OF7_RES2  of7(S_evt      ,res2          ) /*                         */
#define OF7_SSID  of7(S_evt      ,sessid        ) /*                         */
#define OF7_RTID  of7(S_evt      ,rteid         ) /*                         */
#define OF7_EVL   of7(S_evt      ,evl           ) /*                         */
#define OF7_NOD   of7(S_evt      ,node          ) /*                         */
#define OF7_RNOD  of7(S_evt      ,reqnode       ) /*                         */
#define SZEVT     (sizeof(S_evt)                ) /*                         */

/*
   2.3  S_trans transition structure, used by VMK and VMIO
   ---------------------------------------------------

   Cette structure est utilisee par les automates du VMOS ainsi que par ceux
   du VMIO. Chaque table de transition est un tableau de structures S_trans,
   chaque ligne d'une table correspond a une structure S_trans
*/

/* "exec_tran" error or impossible states values ........................... */

#define STNEVER           0x0FFE   /* Next step = never used                 */
#define STERR             0x0FFF   /* Next step = err, impossible transition */

/* "exec_tran" self-generated conditions ....................................*/

#define ENTER             0        /* O_CALL self-generated condition        */
#define EXIT              1        /* O_RET  self-generated condition        */

/* Values for bits b28..31 of field "op" of structure "S_trans" .............*/
/* Those values are the flow control operations code values                  */


#define O_GOTO        0x00000000   /* Goto state "nstate"                    */
#define O_CALL        0x10000000   /* Push state variable and goto "nstate"  */
#define O_RET         0x20000000   /* Pop state variable (RETURN)            */
#define O_EXIT        0x30000000   /* Subroutine exit to error handler       */
#define O_CND         0x40000000   /* Goto then generate a condition         */


typedef struct s_trans                 /* Automaton transition               */
  BEGIN                                /* ---------------------------------- */
    SHORT       code                 ; /* Incoming event code                */
    IWORD       (*proc)()            ; /* Treatment procedure address        */
    UINT        op                   ; /* Flow control + 28 bits parameter   */
    USHORT      nstate               ; /* Automaton next state number        */
  END           S_trans              ; /* ---------------------------------- */

/*
   2.4  S_aut Automaton structure, used by VMK and VMIO
   ----------------------------------------------------

*/

typedef struct s_aut                   /* One automaton data                 */
  BEGIN                                /* ---------------------------------- */
    const S_trans  *atrans           ; /* Transition table address           */
    const USHORT   *alim             ; /* Transition table limits            */
    USHORT         statenb           ; /* Number of automaton states         */
    USHORT         exlwnb            ; /* External Logical Ways number       */
    USHORT         inlwnb            ; /* Internal Logical Ways number       */
    USHORT         stacksz           ; /* Depth of state stacks              */
    USHORT         scndsz            ; /* Depth of the condition stack       */
    USHORT         *astate           ; /* State variable stacks storage addr */
    UCHAR          *apstack          ; /* Stack pointers storage addr        */
    USHORT         *acond            ; /* Conditions stack storage addr      */
    S_evt          *aevt             ; /* Copy of incoming event address     */
  END              S_aut             ; /* ---------------------------------- */

/*
   2.5 Structure Time-out du VMK
   ------------------------------

   La table des temporisateurs en cours du VMK est un tableau de structures
   S_to, on a une structure S_to par time-out en cours
*/

typedef struct s_to                    /* Une entree de la table des TIME-OUT*/
  BEGIN                                /* ---------------------------------- */
    INT         duree                ; /* Duree totale avant echeance en msec*/
    UCHAR       nfa                  ; /* numero de la file d'attente        */
    UCHAR       flags                ; /* 1 si arme par set_uto              */
    USHORT      code                 ; /* Code de l'evt a renvoyer a echeance*/
    INT         res                  ; /* valeur du champ reserve            */
    USHORT      aut                  ; /* Numero automate destinataire du TO */
    USHORT      vl                   ; /* Numero voie logique du destinataire*/
    INT         ident                ; /* Numero d'ordre du timer (24 bits)  */

    INT         compteur             ; /* Temps restant avant echeance en ms */
  END           S_to                 ; /* ---------------------------------- */



/*
   2.6 Structure Time-out du VMIO
   ------------------------------

   La table des temporisateurs en cours du VMIO (moteur simplifie) est un
   tableau de structures S_to_s, on a une structure S_tos par time-out
*/

typedef struct s_to_s                  /* An entry in the TIMER table        */
  BEGIN                                /* ---------------------------------- */
    UINT        ident                ; /* Timer identifier                   */
    USHORT      aut                  ; /* Requesting automaton number        */
    USHORT      vl                   ; /* Requesting logical way number      */
    USHORT      code                 ; /* Event code sent on timeout         */
    USHORT      link                 ; /* Pointer to the next running timer  */
    INT         date                 ; /* Expiration date                    */
  END           S_to_s               ; /* ---------------------------------- */


/*
   2.7 Structure Route du VMK et du VMIO
   -------------------------------------

   La table de routage des evenements du VMK est une table dont chaque
   entree est une structure Route.
*/

typedef struct                         /* A route in the routing table       */
  BEGIN                                /* ---------------------------------- */
    UINT        ident                ; /* Route identifier                   */
    USHORT      aut                  ; /* Destination automaton          (E) */
    USHORT      vl                   ; /* Destination logical way        (E) */
    USHORT      emet                 ; /* Sending automaton              (E) */
    USHORT      evl                  ; /* Sending logical way            (E) */
                                       /*                                    */
    USHORT      code1                ; /* Minimun event code             (E) */
    USHORT      code2                ; /* Maximum event code             (E) */
    UINT        res1                 ; /* Minimum reserve field value    (E) */
    UINT        res2                 ; /* Maximum reserve field value    (E) */
                                       /*                                    */
    USHORT      flags                ; /* Priority +Grab +Normal/Defaut  (E) */
    USHORT      next                 ; /* Chaining to the next route         */
  END           Route                ; /* ---------------------------------- */


/*
   2.8 Structure condition de depot du VMK
   ---------------------------------------

   La table des conditions de depot du VMK est une table dont chaque entree
   est une structure Sess_cond
*/

typedef struct sess_cond               /* Une entre de la table "sess_cond"  */
  BEGIN                                /* ---------------------------------- */
    UINT        iod                  ; /* IOD de la source d'evenements      */
    USHORT      code                 ; /* Code de l'evenement concerne       */
    USHORT      sessid               ; /* Numero de session courant          */
    USHORT      act                  ; /* Bitmap d'actions a effectuer       */
  END           Sess_cond            ; /* ---------------------------------- */
 

/*
   2.9 Structure zone designable sur l'ecran
   -----------------------------------------

*/

typedef struct area                    /* Description d'un rectangle         */
  BEGIN                                /* designable sur l'ecran             */
    INT         aut,vl               ; /* numero d'automate et de voie log.  */
    INT         prio                 ; /* priorite des cases designables     */
    INT         dev,nw               ; /* numero d'ecran de la case design.  */
    INT         x1,y1                ; /* Coin superieur gauche              */
    INT         x2,y2                ; /* Coin inferieur droit               */
    struct area *sgl,*sgr            ; /* liens avec les cases voisines      */
  END           Area                 ; /* Identificteur du type              */



/*
   2.10 Structure de description d'un groupe de taches
   ---------------------------------------------------

   La table statique de description des groupes de taches "task_group[]", qui
   est definie dans "vmk1.c", est un tableau de structures "Task_group"
*/

typedef struct task_grp                /* Description d'un groupe de taches  */
  BEGIN                                /* ---------------------------------- */
    const CHAR *group                ; /* Nom de ce groupe de taches         */
    CHAR        use                  ; /* 1:temps reel, 2:standard           */
    CHAR        autostart            ; /* 0:application 1:init_vmk           */
    IWORD       (*procstart)()       ; /* Procedure demarrage groupe         */
  END           Task_grp             ; /* ---------------------------------- */


/*
   2.11 Structure contexte de haut niveau d'une tache
   --------------------------------------------------

   La table de contexte de taches "task_ctx[]", qui est definie dans le
   fichier "hypspace.h", est un tableau de structures "Task_ctx"

   ATTENTION !!!

   Les proceduresassembleur de basculement de tache utilisent la taille de
   cette structure ainsi que l'offset des champs "proc" et "par".

   Toute modification de cette structure DOIT ABSOLUMENT etre reportee dans
   les procedures"exec_task" des differents hardware.

   !!!
 
   RG le 20/01/2012 : Ouais, super l'avertissement ci-dessus, sauf que S_evt 
   est utilise dans Task_ctx et si S_evt change de taille, Task_ctx change de
   taille et l'assembleur specifique fait ensuite n'importe quoi pour les 
   taches autres que la 0. La bonne solution est d'ajouter des symboles 
   globaux dans vmk1.c et de recuperer leur contenu dans les exec_task en 
   assembleur. On ajoute les constantes glabales :
   . "task_ctx_sz"        la taille d'un element de type "Task_ctx".
   . "task_ctx_of7_proc"  l'offset du champ "proc" au sein de cette structure.
   . "task_ctx_of7_par"   l'offset du champ "par" au sein de cette structure.
*/


typedef struct task_ctx                /* Contexte d'une tache               */
  BEGIN                                /* ---------------------------------- */
/* Description du code de la tache ----------------------------------------- */
    UCHAR          *bpile            ; /* Adresse de base de la pile.        */
    UIWORD          tpile            ; /* Taille de la pile.                 */
    void           *desc             ; /* Description code/DATA/BSS          */
    INT            (*freeproc)()     ; /* Adresse de la proc de liberation.  */

/* Execution d'une transition par le moteur d'automates -------------------- */
    const S_trans  *tra              ; /* Adresse transition en cours        */
    INT            (*proc)()         ; /* Adresse de la procedure d'entree   */
    IWORD           par              ; /* Parametre a passer a "proc"        */
    SHORT           condition        ; /* Modifiee par "relaut"              */

/* Utilises par "schedule" ------------------------------------------------- */
    UCHAR           use              ; /* Utilisation actuelle de ce contexte*/
    UCHAR           group            ; /* Numero du groupe de cette tache    */
    UCHAR           state            ; /* Etat du traitement en cours        */
    UCHAR           numfa            ; /* File depuis laquelle l'evt provient*/
    IWORD           minquantum       ; /* Quantum de temps minimum en msec   */
    UINT            endquantum       ; /* Date de fin du quantum courant     */

/* Utilises pour les taches "utilisateur", pour les VL de AUT_TASK --------- */
    INT            (*entry)(void*)   ; /* Adresse de la procedure d'entree   */
    void           *param            ; /* Parametre a lui passer             */
    S_evt           evtcourant       ; /* Sauvegarde de l'evenement courant  */
    INT             idto             ; /* Id du timer en cours ou 0 si aucun.*/
    USHORT          idx              ; /* Indexes de la pile alloue ou FFFFF */
    UCHAR           nfa              ; /* Numero de la file d'attente 2..15  */
    UCHAR           isem             ; /* Indice du semaphore que l'on attend*/
    UCHAR           next             ; /* Suivante   en attente du semaphore */
    UCHAR           prev             ; /* Precedente en attente du semaphore */
    UCHAR           killed           ; /* Compteur de "delete_task"          */
  END           Task_ctx             ; /* ---------------------------------- */


typedef struct                         /* Task system context.               */
  BEGIN                                /* ---------------------------------- */
    IWORD         *sp                ; /* 0x0000 Stack pointer               */
    void         (*ip)()             ; /* 0x0004 Instruction pointer         */
  END              Task_system       ; /* ---------------------------------- */

    

/* Valeurs du champ "state" .................................................*/

#define ST_NONE     0                  /* Pas de traitement en cours         */
#define ST_RUN      1                  /* Traitement en cours d'execution    */
#define ST_READY    2                  /* Traitement en attente de CPU       */
#define ST_WAIT     3                  /* Traitement en attente d'evenement  */

/*
   2.12 Structure contexte d'un semaphore de tache
   -----------------------------------------------

   La table de contexte de semaphores "task_sem[]", qui est definie dans le
   fichier "hypspace.h", est un tableau de structures "Task_sem"
*/

typedef struct task_sem                /* Un semaphore                       */
  BEGIN                                /* ---------------------------------- */
    volatile USHORT counter          ; /* Valeur du compteur.                */
    UCHAR           first            ; /* Premiere tache en attente.         */
    UCHAR           last             ; /* Derniere tache en attente.         */
    UCHAR           nfa              ; /* File ou le TASK_SIGNAL a ete depose*/
  END               Task_sem         ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________


    3.1. Defined in hypspace.h
*/

/* Constant variable.......................................................*/

extern const UCHAR  maxroute         ; /* Nb total de routes.                */
extern const USHORT sess_cond_max    ; /* Nb total de conditions.            */
extern const SHORT  fai_size         ; /* Nombre d'elements dans "fai_evt[]" */
extern const UCHAR  kbd_max          ; /* Nombre de voies logiques.          */
extern const UCHAR  hmot_nautmax     ; /* Nombre maximum d'automates         */
extern const UCHAR  hmot_ntomax      ; /* 10/50 Nombre maximum de Time - Out */
                                       /*                                    */
extern const UCHAR  ntaskmax         ; /* Nombre maximum de taches.          */
extern const UCHAR  ntsemmax         ; /* Nombre max. de semaphores.         */
extern const UCHAR  nb_stack_sz      ; /* Nb de tailles de piles.            */
                                       /*                                    */

/* Tables ...................................................................*/

extern S_evt        fai_evt      []  ; /* Elements des files internes du VMK */
extern UCHAR        evt_lock     []  ; /* Locks    des files internes du VMK */
extern Route        route        []  ; /* Tableau global des routes.         */
extern Sess_cond    sess_cond    []  ; /* Tableau global des conditions.     */
extern UINT         used_stack   []  ; /* Table allocation des piles         */
extern SHORT        code_evt     []  ; /* codes evts continue_script         */
extern Task_ctx     task_ctx     []  ; /* Contexte des taches.               */
extern USHORT       task_etat    []  ; /* Etat des taches.                   */
extern UCHAR        task_pstack  []  ; /* Stack pointers                     */
                                       /*                                    */
extern Task_sem     task_sem     []  ; /* Semaphores des taches.             */
                                       /*                                    */
extern struct s_to  tabto        []  ; /* Tableau de structures TO           */
                                       /*                                    */

extern S_aut        hmot_aut     []  ; /* Automaton description table        */

/*
    3.1  Defined in vmk1.c
    ----------------------
*/

extern CHAR         numfa            ; /* Numero FA utilisee                 */
extern USHORT       numauto          ; /* Numero automate courant            */
extern USHORT       voielog          ; /* Numero VL courante                 */
extern USHORT       codeevt          ; /* Code de l'evenement courant        */
extern S_evt       *evtcourant       ; /* Copie de adrevt[] pour evt courant */

extern UINT         cptevt           ; /* Compteur d'evement. Il est         */
                                       /* incremente de 1 par putevt_vmk a   */
                                       /* chaque ecritue dans une            */
                                       /* quelconque file d'attente          */

extern S_evt        task_evt         ; /* Evenement courant de AUT_TASK.     */

extern IWORD        idle_sleep       ; /* Comportement en cas d'inactivite   */

extern IWORD        vmos_run         ; /* Flag de fonctionnement du VMOS     */

/*
    4    Prototypes of public procedures
    _____________________________________
*/


/*
   4.1  Prototypes proceduresde la boucle principale du VMK
   ---------------------------------------------------------
*/

  IWORD init_vmk            (CHAR*,CHAR*,UINT*,INT*                          );
  IWORD vmk                 (UINT,S_evt*,UINT*,UINT*                         );
  IWORD end_vmk             (void                                            );


/*
   4.2  Prototypes proceduresautomate de base
   -------------------------------------------
*/


  IWORD iniaut              (INT,const S_trans[],const USHORT[],
                             INT,INT,INT,INT,INT,
                             USHORT[],UCHAR[],USHORT[],S_evt*,INT*           );

  void  relaut              (INT                                             );
  void  pushcond            (INT,INT                                         );
  IWORD putevt_vmk          (IWORD,S_evt*                                    );
  IWORD waitevt_vmk         (UIWORD,UIWORD,UIWORD,UINT,S_evt*                );
  IWORD unlockevt_vmk       (UIWORD,UIWORD,UIWORD                            );
  IWORD clearevt_vmk        (INT,INT,INT,INT,INT*,UINT*                      );
  IWORD alloc_waitevtlist   (INT,UINT*,INT*                                  );
  IWORD free_waitevtlist    (UINT,INT*                                       );
  IWORD add_waitevt         (UINT,INT,INT,INT,INT*                           );
  IWORD del_waitevt         (UINT,INT,INT,INT,INT*                           );
  IWORD clear_waitevtlist   (UINT,INT*                                       );
  IWORD nb_waitevtlist      (UINT,INT*,INT*                                  );
  IWORD chk_evtlist         (UINT,Waitevtlist**                              );
  IWORD chk_typval12        (INT,INT,INT                                     );


/*
   4.3  Prototypes procedurespour les conditions sur sessid
   ---------------------------------------------------------
*/

  IWORD add_sessid_cond     (UINT,UINT,INT,INT*                              );
  IWORD mod_sessid_cond     (UINT,UINT,USHORT*,INT*                          );
  IWORD del_sessid_cond     (UINT,UINT,INT*                                  );
  IWORD get_sessid          (UINT,UINT,USHORT*                               );
  IWORD put_event           (IWORD,S_evt*                                    );
  IWORD del_evtsess         (UINT,UINT                                       );


/*
   4.4  Prototypes procedurespour les routes
   ------------------------------------------
*/

  IWORD add_route           (INT,INT,INT,INT,INT,INT,UINT,UINT,INT,INT*      );
  IWORD list_route          (INT,INT,INT,INT,INT,INT,UINT,UINT,UINT,INT*      ,
                             INT,INT*                                        );
  IWORD del_route           (INT*,INT                                        );
  IWORD set_route_flags     (INT*,INT,INT,INT                                );
  IWORD set_route_iod       (INT*,INT,INT                                    );
  IWORD route_event         (S_evt*                                          );
  IWORD add_uroute          (INT,INT,UINT,UINT,INT,INT*                      );
  IWORD list_uroute         (INT,INT,UINT,UINT,UINT,INT*,INT,INT*            );
  IWORD del_uroute          (INT*,INT                                        );
  IWORD route_uevent        (INT,INT,UCHAR*,INT,INT,INT,INT,INT*             );
  IWORD del_evtroute        (INT                                             );
  IWORD get_route           (INT,Route*                                      );
  IWORD status_route        (IWORD,CHAR*,INT,CHAR*,INT                       );
  IWORD get_event_routing   (INT,CHAR**,CHAR**,INT*,INT*,INT*,CHAR**          ,
                             INT*,INT*,INT*                                  );


/*
   4.5  Prototypes procedurespour les temporisateurs
   --------------------------------------------------
*/

  IWORD set_to              (INT,INT,INT,INT,INT,INT,INT,INT*                );
  IWORD clear_to            (INT,INT,INT                                     );
  IWORD set_uto             (INT,INT,INT,INT,INT*                            );
  IWORD clear_uto           (INT                                             );
  IWORD set_tto             (INT,INT,INT,INT,INT*                            );
  IWORD clear_tto           (INT                                             );
  IWORD clear_autvlto       (INT,INT                                         );


/*
   4.6  Prototypes proceduresvides
   --------------------------------
*/

  IWORD terror              (INT                                             );
  IWORD trien               (INT                                             );


/*
   4.7  Prototypes proceduresde gestion des taches et des piles
   -------------------------------------------------------------
*/

  IWORD create_task         (CHAR*,INT(*)(void*),INT(*)(void),IWORD*,INT,INT  ,
                             void*, INT, INT*                                );
  IWORD delete_task         (INT,INT*                                        );
  IWORD start_task          (INT,INT*                                        );
  IWORD suspend_task        (INT,INT*                                        );
  IWORD exit_task           (void                                            );
  IWORD get_task_id         (INT*,INT*                                       );
  IWORD waitevt_task        (INT[][3],INT,INT,INT,INT*                       );
  IWORD readevt_task        (INT*,INT,INT*                                   );
  IWORD sendevt_task        (INT,INT,INT,UCHAR*,INT,INT,INT,INT,INT*         );
  IWORD get_task_list       (INT,INT*,CHAR*,INT*                             );
  IWORD chk_taskid          (UINT,Task_ctx**                                 );


/*
   4.8  Prototypes procedurespour semaphores
   ------------------------------------------
*/

  IWORD create_semaphore    (INT,INT*                                        );
  IWORD delete_semaphore    (INT,INT*                                        );
  IWORD put_semaphore       (INT,INT*                                        );
  IWORD get_semaphore       (INT,INT*                                        );


/*
   4.9  Prototypes procedurespour mise en veille
   ----------------------------------------------
*/

  IWORD set_idle            (INT                                             );


/*
   4.10 Prototypes procedurespour debugger un logiciel embarque
   -------------------------------------------------------------
*/

  IWORD set_vhalt_att       (INT,INT                                         );
  void  vhalt               (void                                            );
  void  set_evt_debug       (S_evt*                                          );
  IWORD chk_evt_debug       (S_evt*                                          );


/*
   4.11 Prototypes proceduresdestinees au debugger de scripts
   -----------------------------------------------------------
*/

  void  reroute_evt         (IWORD,IWORD,IWORD,IWORD                         );
  void  set_fatmp           (S_evt*,IWORD                                    );
 

/*
   4.12 Prototypes procedurescouche basse du VMK
   ----------------------------------------------
*/

  IWORD main                (INT,CHAR*[]                                     );
  IWORD putevt_fae          (S_evt*                                          );
  IWORD init_hypsys         (void                                            );
  void  end_hypsys          (void                                            );
  void  tim_get             (INT*,INT*,INT*,INT*,INT*,INT*,INT*              );
  void  tim_set             (INT,INT,INT,INT,INT,INT,INT,INT*                );
  void  tim_ticks           (UINT*                                           );
  void  tim_wait_usec       (UINT                                            );
  void  tim_get_usec        (UINT*                                           );
  IWORD init_vmk_spe        (void                                            );
  IWORD init_stack          (IWORD                                           );
  IWORD init_task           (IWORD,IWORD                                     );
  IWORD exec_task           (IWORD                                           );
  IWORD return_task         (IWORD,IWORD                                     );
  IWORD end_task            (IWORD                                           );
  IWORD end_stack           (IWORD                                           );
  IWORD vm_sleep            (IWORD                                           );


/* Prototypes des procedurespubliques du VMK -------------------------------*/

  IWORD time_init           (INT                                             );
  void  time_itmsec         (UINT                                            );
  void  init_fae            (void                                            );
  void  free_fae            (INT,INT                                         );
  INT   ldr_waitevt_fae     (INT,INT,UINT*,S_evt**,INT*,INT*                 );



#endif                                 /* End of MOTEUR_H.                   */
