/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/vmx_posix.h"           */
/*  Constants and definitions for POSIX header file handling.                */
/*                                                                           */
/*  Author : FA                         Last modification : 04/11/2016       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures


*/


#ifndef VMK_POSIX_H                    /* If this file is not already        */
#define VMK_POSIX_H                    /* included, include it.              */


/*
    1    Defines
    ____________
*/

/* New symbols ..............................................................*/

/* Define a set of new symbols for pthread API... This to avoid linking      */
/* errors between Hyperpanel POSIX API and native (Linux, ST...) POSIX API.  */

#define pthread_attr_init               hyp_pthread_attr_init
#define pthread_attr_destroy            hyp_pthread_attr_destroy
#define pthread_attr_getdetachstate     hyp_pthread_attr_getdetachstate
#define pthread_attr_getschedparam      hyp_pthread_attr_getschedparam
#define pthread_attr_getstack           hyp_pthread_attr_getstack
#define pthread_attr_getstackaddr       hyp_pthread_attr_getstackaddr
#define pthread_attr_getstacksize       hyp_pthread_attr_getstacksize
#define pthread_attr_setdetachstate     hyp_pthread_attr_setdetachstate
#define pthread_attr_setschedparam      hyp_pthread_attr_setschedparam
#define pthread_attr_setstack           hyp_pthread_attr_setstack
#define pthread_attr_setstackaddr       hyp_pthread_attr_setstackaddr
#define pthread_attr_setstacksize       hyp_pthread_attr_setstacksize
#define pthread_create                  hyp_pthread_create
#define pthread_join                    hyp_pthread_join
#define pthread_detach                  hyp_pthread_detach
#define pthread_self                    hyp_pthread_self
#define pthread_exit                    hyp_pthread_exit
#define pthread_kill                    hyp_pthread_kill

#define sched_get_priority_max          hyp_sched_get_priority_max
#define sched_get_priority_min          hyp_sched_get_priority_min
#define sched_yield                     hyp_sched_yield

#define pthread_mutex_init              hyp_pthread_mutex_init
#define pthread_mutex_destroy           hyp_pthread_mutex_destroy
#define pthread_mutex_lock              hyp_pthread_mutex_lock
#define pthread_mutex_trylock           hyp_pthread_mutex_trylock
#define pthread_mutex_unlock            hyp_pthread_mutex_unlock

#define sem_init                        hyp_sem_init
#define sem_destroy                     hyp_sem_destroy
#define sem_post                        hyp_sem_post
#define sem_wait                        hyp_sem_wait
#define sem_trywait                     hyp_sem_trywait

#define pthread_cond_init               hyp_pthread_cond_init
#define pthread_cond_destroy            hyp_pthread_cond_destroy
#define pthread_cond_wait               hyp_pthread_cond_wait
#define pthread_cond_timedwait          hyp_pthread_cond_timedwait
#define pthread_cond_signal             hyp_pthread_cond_signal
#define pthread_cond_broadcast          hyp_pthread_cond_broadcast

#define pthread_key_create              hyp_pthread_key_create
#define pthread_key_delete              hyp_pthread_key_delete
#define pthread_setspecific             hyp_pthread_setspecific
#define pthread_getspecific             hyp_pthread_getspecific

#define gettimeofday                    hyp_gettimeofday
#define time                            hyp_time
#define mktime                          hyp_mktime
#define gmtime                          hyp_gmtime
#define localtime                       hyp_localtime
#define localtime_r                     hyp_localtime_r
#define usleep                          hyp_usleep

#define PTHREAD_MUTEX_INITIALIZER  ((pthread_mutex_t) 0xFFFFFFFF)


/* Constants defining POSIX PThread implementation by Hyperpanel ............*/

#define PTHREAD_STACK_MIN   ( 4*1024)  /* Minimum pthread stack size.        */
#define PTHREAD_STACK_DEF   (20*1024)  /* Pthread default stack size.        */
#define PTHREAD_CREATE_DETACHED    0   /* Detached phtread (=>"detachstate") */
#define PTHREAD_CREATE_JOINABLE    1   /* Joinable phtread (=>"detachstate") */
#define SCHED_OTHER                0   /* OS Specific scheduling.            */
#define SCHED_FIFO                 1   /* Real-time scheduling by FIFO.      */
#define SCHED_RR                   2   /* Real-time scheduling by Round-Robin*/
#define PTHREAD_SCOPE_PROCESS      0   /* Thread is an element of a process. */
#define PTHREAD_SCOPE_SYSTEM       1   /* Thread is kernel process.          */
#define PTHREAD_INHERIT_SCHED      0   /* Scheduling policy inherited.       */
#define PTHREAD_EXPLICIT_SCHED     1   /* Scheduling policy from create attr.*/
#define SEM_VALUE_MAX              8   /* Maximum value of a semaphore count.*/
#define SEM_NSEMS_MAX             32   /* Maximum number of semaphores.      */
#define PTHREAD_THREADS_MAX        8   /* Maximum number of tasks.           */
#define PTHREAD_MUTEX_NORMAL       0   /*                                    */
#define PTHREAD_MUTEX_RECURSIVE    1   /*                                    */
#define PTHREAD_KEY_MAX            8   /* Nombre de cle TSD maxi.            */
#define PTHREAD_COND_MAX          16   /* Nombre de variablecondition maxi.*/
#define NBMTX                     40   /* Maximum number of mutexs.          */

#define _POSIX_THREAD_ATTR_STACKADDR      1 /* Thread stack addr can set/get */
#define _POSIX_THREAD_ATTR_STACKSIZE      1 /* Thread stack size can set/get */
#define _POSIX_THREAD_PRIORITY_SCHEDULING 0 /* No scheduling attributes.     */


/* Definitions & structures of Hyperpanel POSIX implementation ..............*/

#ifdef VM_UBUVMOS

#define pthread_t            hyp_pthread_t 
#define pthread_attr_t       hyp_pthread_attr_t 
#define pthread_mutexattr_t  hyp_pthread_mutexattr_t 
#define pthread_mutex_t      hyp_pthread_mutex_t 
#define pthread_key_t        hyp_pthread_key_t 
#define pthread_condattr_t   hyp_pthread_condattr_t 
#define pthread_cond_t       hyp_pthread_cond_t 
#define pthread_sem_t        hyp_pthread_sem_t 
#define sched_param          hyp_sched_param

#endif


/*
    2    Structures and typedefs
    ____________________________
*/

/* Definitions & structures of Hyperpanel POSIX implementation ..............*/

typedef struct                         /* Thread sceduling parameters.       */
  BEGIN                                /* ---------------------------------- */
    UINT         sched_priority      ; /* Thread priority, from PRIO_TSK_MIN */
                                       /* to PRIO_TSK_MAX.                   */
  END            sched_param         ; /* ---------------------------------- */
 
 
typedef struct                         /* Thread attributes.                 */
  BEGIN                                /* ---------------------------------- */
    void        *stackaddr           ; /* Stack smallest address.            */
    UINT         stacksize           ; /* Stack size                         */
    sched_param  schedparam          ; /* Parameters for scheduler.          */
    UCHAR        detachstate         ; /* Detach/Joined state of pthread.    */
    UCHAR        schedpolicy         ; /* Scheduling policy.                 */
    UCHAR        inheritsched        ; /* Scheduling params inherited or not */
    UCHAR        scope               ; /* Scope in term of system process.   */
/* HPL internal parameter .................................................. */
    UINT         state               ; /* Bytes 1-3:Magic number, byte 0:    */
                                       /* 0/1 initialised/associated.        */
  END            pthread_attr_t      ; /* ---------------------------------- */


typedef UINT     pthread_t           ; /* Thread identifier.                 */
typedef UINT     pthread_mutexattr_t ; /* Mutex attributes (not supported).  */
typedef UINT     pthread_mutex_t     ; /* Mutex identifier.                  */
typedef UINT     pthread_key_t       ; /* Key identifier.                    */
typedef UINT     sem_t               ; /* Semaphore identifier.              */
typedef UINT     pthread_condattr_t  ; /* Variable Condition attributes      */
typedef UINT     pthread_cond_t      ; /* Condition variable identifier.     */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


/* POSIX prototypes .........................................................*/

  INT   pthread_attr_init         (pthread_attr_t*                           );
  INT   pthread_attr_destroy      (pthread_attr_t*                           );
  INT   pthread_attr_getdetachstate(const pthread_attr_t*,INT*               );
  INT   pthread_attr_getschedparam (const pthread_attr_t*,sched_param*       );
  INT   pthread_attr_getstack      (const pthread_attr_t*,void**,INT*        );
  INT   pthread_attr_getstackaddr  (const pthread_attr_t*,void**             );
  INT   pthread_attr_getstacksize  (const pthread_attr_t*,INT*               );
  INT   pthread_attr_setdetachstate(pthread_attr_t*,INT                      );
  INT   pthread_attr_setschedparam (pthread_attr_t*, const sched_param*      );
  INT   pthread_attr_setstack     (pthread_attr_t*,void*,INT                 );
  INT   pthread_attr_setstackaddr (pthread_attr_t*,void*                     );
  INT   pthread_attr_setstacksize (pthread_attr_t*,INT                       );
  INT   pthread_join              (pthread_t,void**                          );
  INT   pthread_detach            (pthread_t                                 );
  INT   pthread_self              (void                                      );
  void  pthread_exit              (void*                                     );
  INT   pthread_kill              (pthread_t,INT                             );
  INT   sched_get_priority_max    (INT                                       );
  INT   sched_get_priority_min    (INT                                       );
  INT   sched_yield               (void                                      );
  INT   pthread_mutex_init       (pthread_mutex_t*,const pthread_mutexattr_t*);
  INT   pthread_mutex_destroy     (pthread_mutex_t*                          );
  INT   pthread_mutex_lock        (pthread_mutex_t*                          );
  INT   pthread_mutex_trylock     (pthread_mutex_t*                          );
  INT   pthread_mutex_unlock      (pthread_mutex_t*                          );
  INT   sem_init                  (sem_t*,INT,UINT                           );
  INT   sem_destroy               (sem_t*                                    );
  INT   sem_post                  (sem_t*                                    );
  INT   sem_wait                  (sem_t*                                    );
  INT   sem_trywait               (sem_t*                                    );
  INT   pthread_create            (pthread_t*,const pthread_attr_t*           ,
                                   void*(*)(void*),void*                     );


/* Please note that some modules (hypspace.h, tkdma_71xx.c for example)      */
/* use a limited subset of POSIX.                                            */
/* Following declarations implicitly require sys/types.h to be included.     */
/* This is too risky on VMOS sources, except were it is well masterised, i.e.*/
/* inside vmk_posix.c and modules using it. To allow compilation of other    */
/* modules without sys/types.h, the USE_SYS_TYPES_H define has been added.   */

#ifdef USE_SYS_TYPES_H
  INT   pthread_cond_init   (pthread_cond_t*,pthread_condattr_t*             );
  INT   pthread_cond_destroy(pthread_cond_t*                                 );
  INT   pthread_cond_wait   (pthread_cond_t*,pthread_mutex_t*                );
  INT   pthread_cond_timedwait(pthread_cond_t*,pthread_mutex_t*               ,
                             const struct timespec*                          );
  INT   pthread_cond_signal   (pthread_cond_t*                               );
  INT   pthread_cond_broadcast(pthread_cond_t*                               );
  INT   pthread_key_create  (pthread_key_t*,void(*)(void *)                  );
  INT   pthread_key_delete  (pthread_key_t                                   );
  INT   pthread_setspecific (pthread_key_t,const void*                       );
  void *pthread_getspecific (pthread_key_t                                   );
  INT   gettimeofday        (struct timeval*,void*                           );
  
  time_t time               (time_t*                                         );
  time_t mktime             (struct tm*                                      );
  struct tm *gmtime         (const time_t*                                   );
  struct tm *localtime      (const time_t*                                   );
  struct tm *localtime_r    (const time_t*  ,struct tm*                      );
#endif                                 /* End of USE_SYS_TYPES_H.            */

#endif                                 /* End of VMK_POSIX_H.                */
