#!  /bin/sh
#
#   HyperPanel VMOS environment         File system "os/linux/hyperpanel.sh"
#   Standard environment - VMIO/VMIO - Linux.
#
#   Author : FA                         Last modification : 22/05/2017
#
#   (c) Copyright HyperPanel Lab, All Rights Reserved.
#   IDDN.FR.001.320001.*.S.P.1990.000.10000
#
#


#   Syntax : Called by environment files.


# HyperPanel local environment ------------------------------------------------

  export hexe=$hlocal/$hvm/exe                          # Executables location.
  export hobj=$hlocal/$hvm/obj                          # Objects location.
  export htest=$hlocal/$hvm/test                        # Tests exe location.


# HyperPanel naviguation alias ------------------------------------------------

  alias  app='cd $hhome/user/app'                       # PanTalk apps.
  alias  docosen='cd $hhome/user/docos/en'              # Internal doc.
  alias  docosfr='cd $hhome/user/docos/fr'              # Internal doc.
  alias  exe='cd $hlocal/$hvm/exe'                      # Executables files.
  alias  hardware='cd $hhome/hardware'                  # Hardware directory.
  alias  hhome='cd $hhome'                              # Home directory.
  alias  hlocal='cd $hlocal'                            # Local directory.
  alias  hlog='cd $hhome/log'                           # Log directory.
  alias  hypbo='cd /mnt/hyperpanel/hypbo'               # Back-offices.
  alias  include='cd $hhome/include'                    # Includes files.
  alias  log='cd $hlocal/log'                           # Log directory.
  alias  message='cd $hhome/vmos/message'               # Ascii messages.
  alias  obj='cd $hobj'                                 # Object files.
  alias  os='cd $hhome/os/linux'                        # Operating System env.
  alias  pkg='cd $hhome/package'                        # Packages definition.
  alias  sys='cd $hhome/$hvm'                           # VM resources.
  alias  sysexe='cd $hhome/$hvm/exe'                    # VM executables.
  alias  syssrc='cd $hhome/$hvm/src'                    # VM sources.
                                                        #
  alias  hypdoc='cd /mnt/hyperpanel/hypbo/project/hypdoc' # Documentations.
                                                        #
  function config()   { cd $hhome/config/$1       ; }   # Configuration.
  function hlib()     { cd $hhome/lib/$1          ; }   # Libraries.
  function system()   { cd $hhome/system/$1       ; }   # System modules.
  function user()     { cd $hhome/user/$1         ; }   # User modules.
  function vmio()     { cd $hhome/vmio/$1         ; }   # VMIO source files.
  function vmiodev()  { cd $hhome/vmiodev/$1      ; }   # VMIO source files.
  function vmos()     { cd $hhome/vmos/$1         ; }   # VMOS source files.
  function vmosdev()  { cd $hhome/vmosdev/$1      ; }   # VMOS source files.
                                                        #
  function bewell()   { cd $hhome/user/bewell/$1  ; }   # Bewell modules.
  function gwos()     { cd $hhome/user/gwos/$1    ; }   # GatewayOS modules.
  function hytv()     { cd $hhome/user/hytv/$1    ; }   # HyTV system modules.
  function hytvapp()  { cd $hhome/user/hytvapp/$1 ; }   # HyTV apps.
                                                        #
  function transfer() { cd /mnt/cojyp/transfer/$1 ; }   # Transfer directory.


# Alias for debuffer ----------------------------------------------------------

  alias  hgdb='$hgdb'                                   # Debugger.


# Alias HyperPanel for nfbnx --------------------------------------------------

  alias  nx='cd $nfbnxhome'                             # NX delivery.
  alias  objnx='cd $nfbnxhome/obj/$hvm'                 # NX delivery.
  alias  libnx='cd $nfbnxhome/libraries'                # NX delivery.


# Streams for VMOS simulator --------------------------------------------------

  export hflux=/home/flux                               # Streams location.


# C-language / PanTalk / shell presentation models ----------------------------

  export array=$hhome/vmos/model/docarray.txt           # vi utility.
  export code=$hhome/vmos/model/doccod.txt              # vi utility.
  export com=$hhome/vmos/model/com.c                    # vi utility.
  export end=$hhome/vmos/model/docend.txt               # vi utility.
  export define=$hhome/vmos/model/define.h              # vi utility.
  export diagram=$hhome/vmos/model/docdiag.txt          # vi utility.
  export endif=$hhome/vmos/model/endif.c                # vi utility.
  export glob=$hhome/vmos/model/glob.c                  # vi utility.
  export header=$hhome/vmos/model/header.c              # vi utility.
  export hheader=$hhome/vmos/model/header.sh            # vi utility.
  export ifdef=$hhome/vmos/model/ifdef.c                # vi utility.
  export index=$hhome/vmos/model/docindex.txt           # vi utility.
  export include=$hhome/vmos/model/include.h            # vi utility.
  export list=$hhome/vmos/model/list.c                  # vi utility.
  export method=$hhome/vmos/model/method.isc            # vi utility.
  export mnemo=$hhome/vmos/model/mnemo.txt              # vi utility.
  export tenv=$hhome/vmos/model/tenv.sh                 # vi utility.
  export para=$hhome/vmos/model/para.c                  # vi utility.
  export pcode=$hhome/vmos/model/docpcode.txt           # vi utility.
  export private=$hhome/vmos/model/docpriv.txt          # vi utility.
  export proc=$hhome/vmos/model/docproc.txt             # vi utility.
  export proto=$hhome/vmos/model/proto.h                # vi utility.
  export ptab=$hhome/vmos/model/ptab.isc                # vi utility.
  export script=$hhome/vmos/model/script.isc            # vi utility.
  export tab=$hhome/vmos/model/tab.c                    # vi utility.
  export trans=$hhome/vmos/model/trans.c                # vi utility.
  export typedef=$hhome/vmos/model/typedef.c            # vi utility.
  export xdrv=$hhome/vmos/model/xdrv.c                  # vi utility.


# VMOS start commands ---------------------------------------------------------

  alias dir7105='exe ; snet71x ui -cboot03 -b71 | tee dir7105.log'  # VMOS
  alias dsi1076='exe ; sdsi76x ui -cboot12 -b84 | tee dsi1076.log'  # VMOS
  alias dti1077='exe ; sdti77x ui -cboot04 -b90 | tee dti1077.log'  # VMOS
  alias dtr7105='exe ; sdtr75x ui -cboot09 -b82 | tee dtr7105.log'  # VMOS
  alias sgs7233='exe ; scab72x ui -cboot01 -b83 | tee sgs7233.log'  # VMOS
  alias hd200='exe   ; shd200x ui -cboot05 -b92 | tee hd200.log'    # VMOS
                                                                    #
  alias gowell='exe  ; raspi3    gi -cboot -b00 | tee raspi3.log'   # PIOS

  alias synapos='exe ; syndemo gi -cboot -b00 | tee syndemo.log'  # SynapOS


# Shell configuration ---------------------------------------------------------

  umask 0033                                            # Default file att.
  set    filec                                          # Filename completion.
                                                        #
  alias  dir='ls -alG'                                  # "dir" command.
  alias  ll='ls -lFG'                                   # "ll"  command.
  alias  lla='ls -alFG'                                 # "lla" command.
  alias  lld='ls -ldFG'                                 # "lld" command.
  alias  l='ls -CF'                                     # "l"   command.
  alias  lr='ls -lrtFG'                                 # "lr"  command.
  alias ddc='dd iflag=count_bytes,skip_bytes oflag=seek_bytes'
                                                        # "dd" with byte count. 
  export LESS="-FRnX"                                   # "less" default opt.
  export PAGER="less"                                   # Default pager.
  export EDITOR="vim"                                   # Default editor.
  export HISTSIZE=1000                                  # Historical size.
                                                        #
  if [ $UID = 0 ]; then                                 # Root case:
    export PS1="\[$(xtitle \`pwd\`)\]\h $hvm \w #> "    # Set prompt.
  else                                                  # Other cases:
    export PS1="\[$(xtitle \`pwd\`)\]\h $hvm \w >> "    # Set prompt.
  fi                                                    #


# Trace command ---------------------------------------------------------------

tip() {
  brate=115200
  if [ $# -lt 1 ]; then echo "Please provide TTY interface."; return -1; fi
  case "$1" in
    -*) brate=${1:1}; shift ;;
  esac
  if [ $# -ne 1 ]; then return -2; fi
  socat - "$1",raw,b${brate}
}


# Default directory -----------------------------------------------------------

  cd     $hhome                                         # Default directory.
