/*****************************************************************************/
/*                                                                           */
/*  HyperPanel VMOS                     File "src/space_appcont_stm32py.c"   */
/*  Configurable static data for stm32py target - ByBoard.                   */
/*                                                                           */
/*  Author : JYA                        Last modification : 09/11/2019       */
/*                                                                           */
/*  (c) Copyright HyperPanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


    The module "space_appcont.c" contains all the static RAM data of the VMOS
(application container). These data are arrays wich size can be different,
depends on the target. Each array size is a define which value is define in 
a specific "space_xx.h" file. By using different copies of the "space_xx.h"
file, we can customize the sizeo of the RAM arrays.
    
*/


#include <stdtyp.l>                       /* Definitions and constantes.     */
#include <space_stm32py.h>                /* Sizing of static data.          */
#include <vmk.h>                          /* Task_grp                        */
#include <drv.h>                          /* Sys_driver                      */


/* task_grp[] cont table ------------------------------------------------------

   This const table is needed by the "init_vmk" procedure (vmk1.c). The
   "init_vmk" does a loop on the table entries and the loops stops with
   the first entry with a "use" field is greater than 2.

   The "autostart" functions are here defined as "weak" symbols, thus enabling
   the application de redefine those. The "task_grp" table thus may be
   re-used by different applications that will ne need to define empty
   functions.

*/

int __attribute__((weak)) init_app_fsm(Task_grp *g,char* mod,char*conf) 
  { return 0; }

int __attribute__((weak)) init_app_tsk(Task_grp *g,char* mod,char*conf) 
  { return 0; }

const Task_grp task_grp[] =            /* Definition des groupes de taches   */
  BEGIN                                /* ---------------------------------- */
                                       /* (group use autostart procstart)    */
/*  group      use autostart procstart    ---------------------------------- */
                                       /* Groupes de taches "temps reel" ... */
    "appfsm" , 1 , 1 , init_app_fsm ,  /* VMK User FSM start                 */
    "apptsk" , 1 , 1 , init_app_tsk ,  /* VMK User tasks start               */
                                       /* ---------------------------------- */
                                       /* Groupes de taches "standard" ..... */
                                       /* -                                  */
                                       /* ---------------------------------- */
    ""     , 9 , 9 , HNULL           , /* End of list mark ................. */
  END                                ; /* ---------------------------------- */


/* vmos_driver[] const table --------------------------------------------------

   This const table is needed by the "vmk" procedure (vmk1.c). The "vmk"
   procedure is called from the Application container main loop for each
   external event (comming from the IO container. The first step of "vmk" is
   to call an event procedure: vmos_driver[major]->drv_evt(major,evt) where
   "major" is the vvalue of bits b16..23 of the "reserve" field of the incoming
   event.

   Entry 0 of "vmos_driver[]" MUST be "sysevt_driver", in other words the
   major number for "sysevt_driver" MUST be 0. This is because by convention
   all the system drivers of the IO containter are using a "dummy" IOD value
   that has the form 0xFF00ddss", so with a 00 major number.

#PRIVATE ----------------------------------------------------------------------

   ATTENTION : Un cretin a cree une table nommee "drv_itf_ctx" dans le fichier
   "board_xyz.c", laquelle utilise aussi le major number comme indice. Il
    faut donc modifier en meme temps "vmos_driver" et "drv_itf_ctx", en
    attendant que je recode correctement et que le major redevienne un indice
    de la seule table vmos_driver[]

#END --------------------------------------------------------------------------

*/


extern const Sys_driver sysevt_driver; /*                                    */
extern const Sys_driver membuf_driver; /*                                    */
extern const Sys_driver eep_driver   ; /*                                    */
extern const Dps_driver asy_driver   ; /*                                    */
extern const Dps_driver gpio_driver  ; /*                                    */
extern const Dps_driver led_driver   ; /*                                    */
extern const Dps_driver kbditf_driver; /*                                    */
extern const Dps_driver dac_driver   ; /*                                    */
extern const Dps_driver adc_driver   ; /*                                    */


const Sys_driver* const vmos_driver[]= /* Drivers table for stm32m3.         */
  BEGIN                                /* System drivers (no tags) --------- */
    (Sys_driver*)&sysevt_driver      , /* 0 MUST be entry 0                  */
    (Sys_driver*)&membuf_driver      , /* 1                                  */
    (Sys_driver*)&eep_driver         , /* 2                                  */
                                       /* Character drivers (tags) --------- */
    (Sys_driver*)&asy_driver         , /* 3                                  */
    (Sys_driver*)&gpio_driver        , /* 4                                  */
    (Sys_driver*)&led_driver         , /* 5                                  */
    (Sys_driver*)&kbditf_driver      , /* 6                                  */
    (Sys_driver*)&dac_driver         , /* 7                                  */
    (Sys_driver*)&adc_driver         , /* 8                                  */
    HNULL                            , /* 9                                  */
  END                                ; /*                                    */
                                       /*                                    */

#include <../../vmos/sys/space_appcont.c> /* VMOS static data.               */
